//********************************************************************************************************************************
//*                                                                                                                              *
//*     Dynamic Free Image Library Loader ANSI & Unicode version  3delite 2007-2024                                             *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/index.html                                                      *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit FreeImageLoader;

//* Uncomment the following line (remove the dot ".") to build with VCL graphics support
{$DEFINE USEVCLGRAPHICS}

//* Uncomment the following line (remove the dot ".") to build with Graphics32
{$DEFINE USEGR32}

interface

Uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    {$IFDEF USEVCLGRAPHICS}
    VCL.Graphics,
    {$ENDIF}
    {$IFDEF USEGR32}
    GR32,
    GR32_Backends,
    {$ENDIF}
    FreeImage;

    function InitFreeImageLibrary: Boolean;
    function FreeFreeImageLibrary: Boolean;
    function FreeImageGetFileFormat(FileName: String): FREE_IMAGE_FORMAT;
    {$IFDEF USEVCLGRAPHICS}
    function FreeImageLoadImage(FileName: String; var PicBitmap: TBitmap; Handle: THandle): Boolean; overload;
    function FreeImageLoadImage(FileName: String; var Bitmap: TBitmap; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BPP: longword; Handle: THandle): Boolean; overload;
    function FreeImageLoadImageFromHandleWithRescaleBitmap(BitmapHandle: HBitmap; var PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; Handle: THandle): Boolean;
    function FreeImageLoadImageWithRescaleBitmap(FileName: String; PicBitmap: TBitmap; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
    function FreeImageLoadImageFromFreeImageWithRescaleBitmap(FIBitmapHandle: PFIBITMAP; var PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; Handle: THandle): Boolean;
    function FreeImageCreateImageFromHandle(BitmapHandle: HBitmap; Handle: THandle): PFIBITMAP;
    function FreeImageSaveImage(Picture: HBitmap; FileName: String; Format: FREE_IMAGE_FORMAT; Flags: Integer; ConvertTo24Bit: Boolean = False): Boolean;
    function FreeImageLoadImageFromMemoryTBitmap(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap; Handle: THandle): Boolean; overload;
    function FreeImageRescaleImage(var Picture: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: Integer; KeepAspectRatio: Boolean): Boolean;
    function FreeImageLoadImageFromMemoryWithRescaleTBitmap(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;
    {$ENDIF}
    {$IFDEF USEGR32}
    function FreeImageLoadImageTBitmap32(FileName: String; PicBitmap: TBitmap32; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; JPEGResizeSize: Integer = 0): Boolean; overload;
    function FreeImageLoadImageTBitmap32BPP(FileName: String; PicBitmap: TBitmap32; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BitDepth: Integer; JPEGResizeSize: Integer = 0): Boolean;
    function FreeImageLoadImageWithRescale(FileName: String; PicBitmap: TBitmap32; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
    function FreeImageLoadImageFromHandle(BitmapHandle: HBitmap; var PicBitmap: TBitmap32; Handle: THandle = 0): Boolean;
    function FreeImageLoadImageFromHandleWithRescale(BitmapHandle: HBitmap; var PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; Handle: THandle): Boolean;
    function FreeImageLoadImageFromFreeImageWithRescale(FIBitmapHandle: PFIBITMAP; var PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; Handle: THandle): Boolean;
    function FreeImageLoadImageFromMemoryTBitmap32(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; Handle: THandle; JPEGResizeSize: Integer = 0): Boolean; overload;
    function FreeImageLoadImageFromMemoryTBitmap32WithRescale(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;
    function FreeImageLoadImageFromMemoryTBitmap32WithRescaleAndInfo(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle; var OriginalWidth: Integer; var OriginalHeight: Integer; var DetectedFormat: FREE_IMAGE_FORMAT; var BitDepth: Cardinal): Boolean;
    function FreeImageAdjustGamma(Picture: HBitmap; Bitmap32: TBitmap32; Gamma: Double): Boolean;
    function FreeImageLoadImageWithRescaleIR(FileName: String; PicBitmap: TBitmap32; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
    {$ENDIF}

    function FreeImageSaveImageToMemory(Picture: HBitmap; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream): Boolean;
    function FreeImageResizeSaveImageToMemory(Picture: HBitmap; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean; var ResizedWidth: Integer; var ResizedHeight: Integer): Boolean;
    function FreeImageLoadResizeSaveImageToMemory(FileName: String; SaveFormat: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var ResizedWidth: Integer; var ResizedHeight: Integer; var BitDepth: Cardinal): Boolean;
    function FreeImageLoadFromMemoryResizeSaveImageToMemory(Location: Pointer; MemorySize: Int64; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean): Boolean;

    function FreeImageSaveImageQuantize(Picture: HBitmap; FileName: String; Format: FREE_IMAGE_FORMAT; Flags: Integer): Boolean;

    function FreeImageGetJPEGDimensions(FileName: String; var OriginalWidth: Integer; var OriginalHeight: Integer): Boolean;
    function FreeImageGetImageAttributes(FileName: String; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BitDepth: Cardinal; JPEGResizeSize: Integer = 0): Boolean;

    function FreeImageLoadImageFromMemoryBitmapHandleWithRescale(Location: Pointer; MemorySize: Int64; var BitmapHandle: HBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;

    function FreeImageLoadImageFromMemoryWithRescaleAndSave(Location: Pointer; MemorySize: Int64; FileName: String; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; SaveFormat: FREE_IMAGE_FORMAT): Boolean;

    function FreeImageQuantize(InPicture: HBitmap; var OutPicture: HBitmap; Quantize: FREE_IMAGE_QUANTIZE = FIQ_NNQUANT): Boolean;

const
    FIDLL = 'FreeImage.dll';

    {$IFDEF WIN64}
    {$IFDEF UNICODE}
    NAME_FreeImage_GetFileType = 'FreeImage_GetFileTypeU';
    NAME_FreeImage_GetFIFFromFilename = 'FreeImage_GetFIFFromFilenameU';
    NAME_FreeImage_Load = 'FreeImage_LoadU';
    NAME_FreeImage_Save = 'FreeImage_SaveU';
    {$ELSE}
    NAME_FreeImage_GetFileType = 'FreeImage_GetFileType';
    NAME_FreeImage_GetFIFFromFilename = 'FreeImage_GetFIFFromFilename';
    NAME_FreeImage_Load = 'FreeImage_Load';
    NAME_FreeImage_Save = 'FreeImage_Save';
    {$ENDIF}
    NAME_FreeImage_GetInfoHeader = 'FreeImage_GetInfoHeader';
    NAME_FreeImage_GetInfo = 'FreeImage_GetInfo';
    NAME_FreeImage_GetBPP = 'FreeImage_GetBPP';
    NAME_FreeImage_GetBits = 'FreeImage_GetBits';
    NAME_FreeImage_Unload = 'FreeImage_Unload';
    NAME_FreeImage_Initialise = 'FreeImage_Initialise';
    NAME_FreeImage_DeInitialise = 'FreeImage_DeInitialise';
    NAME_FreeImage_Allocate = 'FreeImage_Allocate';
    NAME_FreeImage_GetColorsUsed = 'FreeImage_GetColorsUsed';
    NAME_FreeImage_GetHeight = 'FreeImage_GetHeight';
    NAME_FreeImage_GetWidth = 'FreeImage_GetWidth';
    NAME_FreeImage_Rescale = 'FreeImage_Rescale';
    NAME_FreeImage_OpenMemory = 'FreeImage_OpenMemory';
    NAME_FreeImage_CloseMemory = 'FreeImage_CloseMemory';
    NAME_FreeImage_LoadFromMemory = 'FreeImage_LoadFromMemory';
    NAME_FreeImage_GetFileTypeFromMemory = 'FreeImage_GetFileTypeFromMemory';
    NAME_FreeImage_SaveToMemory = 'FreeImage_SaveToMemory';
    NAME_FreeImage_AcquireMemory = 'FreeImage_AcquireMemory';
    NAME_FreeImage_GetMetadata = 'FreeImage_GetMetadata';
    NAME_FreeImage_GetTagValue = 'FreeImage_GetTagValue';
    NAME_FreeImage_ConvertTo24Bits = 'FreeImage_ConvertTo24Bits';
    NAME_FreeImage_ColorQuantize = 'FreeImage_ColorQuantize';
    NAME_FreeImage_AdjustGamma = 'FreeImage_AdjustGamma';
    NAME_FreeImage_ConvertTo32Bits = 'FreeImage_ConvertTo32Bits';
    {$ELSE}
    {$IFDEF UNICODE}
    NAME_FreeImage_GetFileType = '_FreeImage_GetFileTypeU@8';
    NAME_FreeImage_GetFIFFromFilename = '_FreeImage_GetFIFFromFilenameU@4';
    NAME_FreeImage_Load = '_FreeImage_LoadU@12';
    NAME_FreeImage_Save = '_FreeImage_SaveU@16';
    {$ELSE}
    NAME_FreeImage_GetFileType = '_FreeImage_GetFileType@8';
    NAME_FreeImage_GetFIFFromFilename = '_FreeImage_GetFIFFromFilename@4';
    NAME_FreeImage_Load = '_FreeImage_Load@12';
    NAME_FreeImage_Save = '_FreeImage_Save@16';
    {$ENDIF}
    NAME_FreeImage_GetInfoHeader = '_FreeImage_GetInfoHeader@4';
    NAME_FreeImage_GetInfo = '_FreeImage_GetInfo@4';
    NAME_FreeImage_GetBPP = '_FreeImage_GetBPP@4';
    NAME_FreeImage_GetBits = '_FreeImage_GetBits@4';
    NAME_FreeImage_Unload = '_FreeImage_Unload@4';
    NAME_FreeImage_Initialise = '_FreeImage_Initialise@4';
    NAME_FreeImage_DeInitialise = '_FreeImage_DeInitialise@0';
    NAME_FreeImage_Allocate = '_FreeImage_Allocate@24';
    NAME_FreeImage_GetColorsUsed = '_FreeImage_GetColorsUsed@4';
    NAME_FreeImage_GetHeight = '_FreeImage_GetHeight@4';
    NAME_FreeImage_GetWidth = '_FreeImage_GetWidth@4';
    NAME_FreeImage_Rescale = '_FreeImage_Rescale@16';
    NAME_FreeImage_OpenMemory = '_FreeImage_OpenMemory@8';
    NAME_FreeImage_CloseMemory = '_FreeImage_CloseMemory@4';
    NAME_FreeImage_LoadFromMemory = '_FreeImage_LoadFromMemory@12';
    NAME_FreeImage_GetFileTypeFromMemory = '_FreeImage_GetFileTypeFromMemory@8';
    NAME_FreeImage_SaveToMemory = '_FreeImage_SaveToMemory@16';
    NAME_FreeImage_AcquireMemory = '_FreeImage_AcquireMemory@12';
    NAME_FreeImage_GetMetadata = '_FreeImage_GetMetadata@16';
    NAME_FreeImage_GetTagValue = '_FreeImage_GetTagValue@4';
    NAME_FreeImage_ConvertTo24Bits = '_FreeImage_ConvertTo24Bits@4';
    NAME_FreeImage_ColorQuantize = '_FreeImage_ColorQuantize@8';
    NAME_FreeImage_AdjustGamma = '_FreeImage_AdjustGamma@12';
    NAME_FreeImage_ConvertTo32Bits = '_FreeImage_ConvertTo32Bits@4';
    {$ENDIF}

type
    EFIUnknownFileFormat = class (Exception);

type
    t_FreeImage_GetFileType = function (const filename: PChar; size: Integer): FREE_IMAGE_FORMAT; stdcall;
    t_FreeImage_GetFIFFromFilename = function (const fname: PChar): FREE_IMAGE_FORMAT; stdcall;
    t_FreeImage_Load = function (fif: FREE_IMAGE_FORMAT; const filename: PChar; flags: integer = 0): PFIBITMAP; stdcall;
    t_FreeImage_GetInfoHeader = function (dib: PFIBITMAP): PBITMAPINFOHEADER; stdcall;
    t_FreeImage_GetInfo = function (var dib: FIBITMAP): PBITMAPINFO; stdcall;
    t_FreeImage_GetBPP = function (dib: PFIBITMAP): Cardinal; stdcall;
    t_FreeImage_GetBits = function (dib: PFIBITMAP): PByte; stdcall;
    t_FreeImage_Unload = procedure (dib: PFIBITMAP); stdcall;
    t_FreeImage_Initialise = procedure (load_local_plugins_only : boolean = False); stdcall;
    t_FreeImage_DeInitialise =  procedure; stdcall;
    t_FreeImage_Save = function (fif: FREE_IMAGE_FORMAT; dib: PFIBITMAP; filename: PWideChar; flags: Integer = 0): Boolean; stdcall;
    t_FreeImage_Allocate = function (width, height, bpp: Integer; red_mask: Cardinal = 0; green_mask: Cardinal = 0; blue_mask: Cardinal = 0): PFIBITMAP; stdcall;
    t_FreeImage_GetColorsUsed = function (dib: PFIBITMAP): Cardinal; stdcall;
    t_FreeImage_GetHeight = function (dib: PFIBITMAP): Cardinal; stdcall;
    t_FreeImage_GetWidth = function (dib: PFIBITMAP): Cardinal; stdcall;
    t_FreeImage_Rescale = function (dib: PFIBITMAP; dst_width, dst_height: Integer; filter: FREE_IMAGE_FILTER): PFIBITMAP; stdcall;
    t_FreeImage_OpenMemory = function (data: PByte = nil; size_in_bytes: DWORD = 0): PFIMEMORY; stdcall;
    t_FreeImage_CloseMemory = procedure (stream: PFIMEMORY); stdcall;
    t_FreeImage_LoadFromMemory = function (fif: FREE_IMAGE_FORMAT; stream: PFIMEMORY; flags: Integer = 0): PFIBITMAP; stdcall;
    t_FreeImage_GetFileTypeFromMemory = function (stream: PFIMEMORY; size: Integer = 0): FREE_IMAGE_FORMAT; stdcall;
    t_FreeImage_SaveToMemory = function (fif: FREE_IMAGE_FORMAT; dib: PFIBITMAP; stream: PFIMEMORY; flags: Integer = 0): Boolean; stdcall;
    t_FreeImage_AcquireMemory = function (stream: PFIMEMORY; var data: PByte; var size_in_bytes: DWORD): Boolean; stdcall;
    t_FreeImage_GetMetaData = function (model: FREE_IMAGE_MDMODEL; dib: PFIBITMAP; key: PAnsiChar; var tag: PFITAG): Boolean; stdcall;
    t_FreeImage_GetTagValue = function (tag: PFITAG): Pointer; stdcall;
    t_FreeImage_ColorQuantize = function (dib: PFIBITMAP; quantize: FREE_IMAGE_QUANTIZE): PFIBITMAP; {$IFDEF MSWINDOWS} stdcall {$ELSE} cdecl {$ENDIF};
    t_FreeImage_ConvertTo24Bits = function (dib: PFIBITMAP): PFIBITMAP; {$IFDEF MSWINDOWS} stdcall {$ELSE} cdecl {$ENDIF};
    t_FreeImage_ConvertTo32Bits = function (dib: PFIBITMAP): PFIBITMAP; {$IFDEF MSWINDOWS} stdcall {$ELSE} cdecl {$ENDIF};
    t_FreeImage_AdjustGamma = function (dib: PFIBITMAP; gamma: Double): LongBool; stdcall;
    
type
    TBGRA32 = packed record
        B, G, R, A: Byte;
    end;
    TBGRA32Array = packed array[0..MaxInt div SizeOf(TBGRA32)-1] of TBGRA32;
    PBGRA32Array = ^TBGRA32Array;

var
    FIL_FreeImage_GetFileType: t_FreeImage_GetFileType;
    FIL_FreeImage_GetFIFFromFilename: t_FreeImage_GetFIFFromFilename;
    FIL_FreeImage_Load:	t_FreeImage_Load;
    FIL_FreeImage_GetInfoHeader: t_FreeImage_GetInfoHeader;
    FIL_FreeImage_GetInfo: t_FreeImage_GetInfo;
    FIL_FreeImage_GetBPP: t_FreeImage_GetBPP;
    FIL_FreeImage_GetBits: t_FreeImage_GetBits;
    FIL_FreeImage_Unload: t_FreeImage_Unload;
    FIL_FreeImage_Initialise: t_FreeImage_Initialise;
    FIL_FreeImage_DeInitialise:	t_FreeImage_DeInitialise;
    FIL_FreeImage_Save: t_FreeImage_Save;
    FIL_FreeImage_Allocate: t_FreeImage_Allocate;
    FIL_FreeImage_GetColorsUsed: t_FreeImage_GetColorsUsed;
    FIL_FreeImage_GetHeight: t_FreeImage_GetHeight;
    FIL_FreeImage_GetWidth: t_FreeImage_GetWidth;
    FIL_FreeImage_Rescale: t_FreeImage_Rescale;
    FIL_FreeImage_OpenMemory: t_FreeImage_OpenMemory;
    FIL_FreeImage_CloseMemory: t_FreeImage_CloseMemory;
    FIL_FreeImage_LoadFromMemory: t_FreeImage_LoadFromMemory;
    FIL_FreeImage_GetFileTypeFromMemory: t_FreeImage_GetFileTypeFromMemory;
    FIL_FreeImage_SaveToMemory: t_FreeImage_SaveToMemory;
    FIL_FreeImage_AcquireMemory: t_FreeImage_AcquireMemory;
    FIL_FreeImage_GetMetaData: t_FreeImage_GetMetaData;
    FIL_FreeImage_GetTagValue: t_FreeImage_GetTagValue;
    FIL_FreeImage_ColorQuantize: t_FreeImage_ColorQuantize;
    FIL_FreeImage_ConvertTo24Bits: t_FreeImage_ConvertTo24Bits;
    FIL_FreeImage_ConvertTo32Bits: t_FreeImage_ConvertTo32Bits;
    FIL_FreeImage_AdjustGamma: t_FreeImage_AdjustGamma;

var
    FreeImageDLLHandle: THandle = 0;
    FreeImageDLLLoaded: Boolean;

implementation

function InitFreeImageLibrary: Boolean;
begin
    FreeImageDLLHandle := SafeLoadLibrary(FIDLL);
    Result := FreeImageDLLHandle <> 0;
    if Result then begin
        FIL_FreeImage_GetFileType := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetFileType));
        FIL_FreeImage_GetFIFFromFilename := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetFIFFromFilename));
        FIL_FreeImage_Load := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Load));
        FIL_FreeImage_GetInfoHeader := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetInfoHeader));
        FIL_FreeImage_GetInfo := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetInfo));
        FIL_FreeImage_GetBPP := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetBPP));
        FIL_FreeImage_GetBits := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetBits));
        FIL_FreeImage_Unload := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Unload));
        FIL_FreeImage_Initialise := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Initialise));
        FIL_FreeImage_DeInitialise := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_DeInitialise));
        FIL_FreeImage_Save := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Save));
        FIL_FreeImage_Allocate := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Allocate));
        FIL_FreeImage_GetColorsUsed := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetColorsUsed));
        FIL_FreeImage_GetHeight := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetHeight));
        FIL_FreeImage_GetWidth := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetWidth));
        FIL_FreeImage_Rescale := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_Rescale));
        FIL_FreeImage_OpenMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_OpenMemory));
        FIL_FreeImage_CloseMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_CloseMemory));
        FIL_FreeImage_LoadFromMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_LoadFromMemory));
        FIL_FreeImage_GetFileTypeFromMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetFileTypeFromMemory));
        FIL_FreeImage_SaveToMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_SaveToMemory));
        FIL_FreeImage_AcquireMemory := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_AcquireMemory));
        FIL_FreeImage_GetMetaData := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetMetadata));
        FIL_FreeImage_GetTagValue := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_GetTagValue));
        FIL_FreeImage_ColorQuantize := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_ColorQuantize));
        FIL_FreeImage_ConvertTo24Bits := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_ConvertTo24Bits));
        FIL_FreeImage_ConvertTo32Bits := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_ConvertTo32Bits));
        FIL_FreeImage_AdjustGamma := GetProcAddress(FreeImageDLLHandle, PChar(NAME_FreeImage_AdjustGamma));
    end;
    if (@FIL_FreeImage_GetFileType = nil)
    OR (@FIL_FreeImage_GetFIFFromFilename = nil)
    OR (@FIL_FreeImage_Load = nil)
    OR (@FIL_FreeImage_GetInfoHeader = nil)
    OR (@FIL_FreeImage_GetInfo = nil)
    OR (@FIL_FreeImage_GetBPP = nil)
    OR (@FIL_FreeImage_GetBits = nil)
    OR (@FIL_FreeImage_Unload = nil)
    OR (@FIL_FreeImage_Initialise = nil)
    OR (@FIL_FreeImage_DeInitialise = nil)
    OR (@FIL_FreeImage_Save = nil)
    OR (@FIL_FreeImage_Allocate = nil)
    OR (@FIL_FreeImage_GetColorsUsed = nil)
    OR (@FIL_FreeImage_GetHeight = nil)
    OR (@FIL_FreeImage_GetWidth = nil)
    OR (@FIL_FreeImage_Rescale = nil)
    OR (@FIL_FreeImage_OpenMemory = nil)
    OR (@FIL_FreeImage_CloseMemory = nil)
    OR (@FIL_FreeImage_LoadFromMemory = nil)
    OR (@FIL_FreeImage_GetFileTypeFromMemory = nil)
    OR (@FIL_FreeImage_SaveToMemory = nil)
    OR (@FIL_FreeImage_AcquireMemory = nil)
    OR (@FIL_FreeImage_GetMetaData = nil)
    OR (@FIL_FreeImage_GetTagValue = nil)
    OR (@FIL_FreeImage_ColorQuantize = nil)
    OR (@FIL_FreeImage_ConvertTo24Bits = nil)
    OR (@FIL_FreeImage_ConvertTo32Bits = nil)
    OR (@FIL_FreeImage_AdjustGamma = nil)
        then Result := False;
    if Result
        then FreeImageDLLLoaded := True;
end;

function FreeFreeImageLibrary: Boolean;
begin
    Result := False;
    if FreeImageDLLHandle <> 0 then begin
        Result := FreeLibrary(FreeImageDLLHandle);
        if Result
            then FreeImageDLLLoaded := False;
    end;
end;

function FreeImageGetFileFormat(FileName: String): FREE_IMAGE_FORMAT;
var
    Fext : string;
begin
    Result := FIF_UNKNOWN;
    if NOT FreeImageDLLLoaded
        then Exit;
    Fext := UpperCase(ExtractFileExt(FileName));
    if (Fext = '.ANI')
    OR (Fext = '.ICO')
    then begin
        Exit;
    end;
    try
        Result := FIL_FreeImage_GetFileType(PChar(FileName), 16);
        // no signature ?
        // try to guess the file format from the file extension
        if Result = FIF_UNKNOWN
            then Result := FIL_FreeImage_GetFIFFromFilename(PChar(FileName));
        if Result = FIF_UNKNOWN then begin
            // Check for types not supported by GetFileType
            Fext := WideUpperCase(ExtractFileExt(FileName));
            if (Fext = '.TGA') or (Fext = '.TARGA')
                then Result := FIF_TARGA
                else if Fext = '.MNG'
                    then Result := FIF_MNG
                    else if Fext = '.PCD'
                        then Result := FIF_PCD
                        else if Fext = '.WBMP'
                            then Result := FIF_WBMP
                            else if Fext = '.CUT'
                                then Result := FIF_CUT;
                                //else raise EFIUnknownFileFormat.Create('The file "' + FileName + '" cannot be displayed because SFM does not recognise the file type.');
        end;
    except
        //*
    end;
end;

{$IFDEF USEVCLGRAPHICS}
function FreeImageLoadImage(FileName: String; var PicBitmap: TBitmap; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PBGRA32Array;
    DC: HDC;
    BPP: longword;
    Flags: Integer;
    Line: PBGRA32Array;
    LineNumber: Integer;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        t := FreeImageGetFileFormat(PChar(FileName));
        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        dib := FIL_FreeImage_Load(t, PChar(FileName), Flags);
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if NOT Assigned(PicBitmap) then begin
            PicBitmap := TBitmap.Create;
        end;

        if BPP = 32 then begin
            PicBitmap.Canvas.Lock;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                PicBitmap.PixelFormat := pf32bit;
                PicBitmap.Transparent := True;
                //PicBitmap.AlphaFormat := afDefined;
                BP := PBGRA32Array(FIL_FreeImage_GetBits(dib));
                LineNumber := 0;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].B := BP[LineNumber * PicBitmap.Width + x].B;
                        Line[x].G := BP[LineNumber * PicBitmap.Width + x].G;
                        Line[x].R := BP[LineNumber * PicBitmap.Width + x].R;
                        Line[x].A := BP[LineNumber * PicBitmap.Width + x].A;
                    end;
                    inc(LineNumber);
                end;
            finally
                PicBitmap.Canvas.UnLock;
            end;
        end else begin
            DC := GetDC(Handle);
            PicBitmap.Canvas.Lock;
            try
                PicBitmap.Assign(nil);
                PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                PicBitmap.PixelFormat := pf32bit;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].A := 255;
                    end;
                end;
            finally
                PicBitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        Result := True;
    except
        //*
    end;
end;

function FreeImageLoadImageFromHandleWithRescaleBitmap(BitmapHandle: HBitmap; var PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    Width: Integer;
    Height: Integer;
    Perc: Double;
    LoadDib: PFIBITMAP;
    DC: HDC;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(BitmapHandle, sizeof(BITMAP), @bm);
    dib := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib);
    dc := GetDC(Handle);
    pdib := dib;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, BitmapHandle, 0, FIL_FreeImage_GetHeight(dib), FIL_FreeImage_GetBits(dib), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(Handle, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib).biClrImportant := nColors;

    try

        if Dib.data = nil
            then Exit;

        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);

        OriginalWidth := Width;
        OriginalHeight := Height;

        //* Keep the aspect
        if (NewWidth / Width) > (NewHeight / Height)
            then Perc := (NewHeight / Height)
            else Perc := (NewWidth / Width);
        //* If smaller don't resize
        //if (Width < NewWidth)
        //AND (Height < NewHeight)
        //then begin
        //    Perc := 1;
        //end;

        LoadDib := dib.data;

        dib := FIL_FreeImage_Rescale(dib, Trunc(Perc * Width), Trunc(Perc * Height), Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);
        PicBitmap.Assign(nil);
        DC := GetDC(Handle);
        PicBitmap.Canvas.Lock;
        try
            PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
        finally
            PicBitmap.Canvas.UnLock;
            ReleaseDC(Handle, DC);
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib);
end;

function FreeImageLoadImageWithRescaleBitmap(FileName: String; PicBitmap: TBitmap; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    DC: HDC;
    Flags: Integer;
    Width: Integer;
    Height: Integer;
    Perc: Double;
    LoadDib: PFIBITMAP;
    Line: PBGRA32Array;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        DetectedFormat := FreeImageGetFileFormat(PChar(FileName));
        if DetectedFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_Load(DetectedFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil then begin
            Exit;
        end;

        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);

        OriginalWidth := Width;
        OriginalHeight := Height;

        //* Keep the aspect
        if KeepAspectRatio then begin

            if (NewWidth / Width) > (NewHeight / Height)
                then Perc := (NewHeight / Height)
                else Perc := (NewWidth / Width);

            //* If smaller don't resize
            if (Width < NewWidth)
            AND (Height < NewHeight)
            then begin
                Perc := 1;
            end;

            NewWidth := Round(Width * Perc);
            NewHeight := Round(Height * Perc);

        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, NewWidth, NewHeight, Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);
        PicBitmap.Canvas.Lock;
        PicBitmap.Assign(nil);
        PicBitmap.Canvas.UnLock;
        DC := GetDC(Handle);
        PicBitmap.Canvas.Lock;
        PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
        PicBitmap.Canvas.UnLock;
        ReleaseDC(Handle, DC);

        PicBitmap.Canvas.Lock;
        try
            PicBitmap.PixelFormat := pf32bit;
            for y := PicBitmap.Height - 1 downto 0 do begin
                Line := PicBitmap.ScanLine[y];
                for x := 0 to PicBitmap.Width - 1 do begin
                    Line[x].A := 255;
                end;
            end;
        finally
            PicBitmap.Canvas.UnLock;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromFreeImageWithRescaleBitmap(FIBitmapHandle: PFIBITMAP; var PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    x, y: integer;
    DC: HDC;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    Line: PBGRA32Array;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;

    try

        if FIBitmapHandle = nil
            then Exit;

        dib := FIL_FreeImage_Rescale(FIBitmapHandle, NewWidth, NewHeight, Resampler);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);
        PicBitmap.Assign(nil);
        DC := GetDC(Handle);
        PicBitmap.Canvas.Lock;
        PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
        PicBitmap.Canvas.UnLock;
        ReleaseDC(Handle, DC);

        PicBitmap.Canvas.Lock;
        try
            PicBitmap.PixelFormat := pf32bit;
            for y := PicBitmap.Height - 1 downto 0 do begin
                Line := PicBitmap.ScanLine[y];
                for x := 0 to PicBitmap.Width - 1 do begin
                    Line[x].A := 255;
                end;
            end;
        finally
            PicBitmap.Canvas.UnLock;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageCreateImageFromHandle(BitmapHandle: HBitmap; Handle: THandle): PFIBITMAP;
var
    dib: PFIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    DC: HDC;
begin
    Result := nil;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(BitmapHandle, sizeof(BITMAP), @bm);
    dib := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib);
    dc := GetDC(Handle);
    pdib := dib;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, BitmapHandle, 0, FIL_FreeImage_GetHeight(dib), FIL_FreeImage_GetBits(dib), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(Handle, dc);
    // Restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib).biClrImportant := nColors;
    Result := dib;
end;

function FreeImageSaveImage(Picture: HBitmap; FileName: String; Format: FREE_IMAGE_FORMAT; Flags: Integer; ConvertTo24Bit: Boolean = False): Boolean;
var
    dib: FIBITMAP;
    pdib, pdib24: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(Picture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // Restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;
    //* Convert to 24 bit
    if ConvertTo24Bit
    OR (Format = FIF_JPEG)
    OR (Format = FIF_GIF)
    then begin
        pdib24 := FIL_FreeImage_ConvertTo24Bits(dib.data);
        FIL_FreeImage_Unload(dib.data);
        dib.data := pdib24;
    end;
    //* Convert to 8 bit
    if Format = FIF_GIF then begin
        pdib24 := FIL_FreeImage_ColorQuantize(dib.data, FIQ_NNQUANT);
        FIL_FreeImage_Unload(dib.data);
        dib.data := pdib24;
    end;
    //* Save
    Result := FIL_FreeImage_Save(Format, dib.data, PWideChar(FileName), Flags);
    // Don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib.data);
end;

function FreeImageRescaleImage(var Picture: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: Integer; KeepAspectRatio: Boolean): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    x, y: integer;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    Line: PBGRA32Array;
    Width: Integer;
    Height: Integer;
    Perc: Double;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(Picture.Handle, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture.Handle, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;

    //* Rescale
    Width := FIL_FreeImage_GetWidth(dib.data);
    Height := FIL_FreeImage_GetHeight(dib.data);
    if KeepAspectRatio then begin
        //* Keep the aspect
        if (NewWidth / Width) > (NewHeight / Height)
            then Perc := (NewHeight / Height)
            else Perc := (NewWidth / Width);

        //* If smaller don't resize
        if (Width < NewWidth)
        AND (Height < NewHeight)
        then begin
            Perc := 1;
        end;

        NewWidth := Round(Width * Perc);
        NewHeight := Round(Height * Perc);
    end;

    try
        pdib := FIL_FreeImage_Rescale(pdib, NewWidth, NewHeight, Resampler);

        PBH := FIL_FreeImage_GetInfoHeader(pdib);
        PBI := FIL_FreeImage_GetInfo(pdib^);

        Picture.Assign(nil);
        DC := GetDC(0);
        Picture.Canvas.Lock;
        Picture.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(pdib)), PBI^, DIB_RGB_COLORS);
        Picture.Canvas.UnLock;
        ReleaseDC(0, DC);

        Picture.Canvas.Lock;
        try
            Picture.PixelFormat := pf32bit;
            for y := Picture.Height - 1 downto 0 do begin
                Line := Picture.ScanLine[y];
                for x := 0 to Picture.Width - 1 do begin
                    Line[x].A := 255;
                end;
            end;
        finally
            Picture.Canvas.UnLock;
        end;

        FIL_FreeImage_Unload(pdib);
        if (Picture.Width > 0)
        AND (Picture.Height > 0)
        then begin
            Result := True;
        end;
    except
        Result := False;
    end;
end;

function FreeImageLoadImageFromMemoryTBitmap(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PBGRA32Array;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    MemoryHandle: PFIMEMORY;
    Line: PBGRA32Array;
    LineNumber: Integer;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin

            PicBitmap.Canvas.Lock;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                PicBitmap.PixelFormat := pf32bit;
                PicBitmap.Transparent := True;
                PicBitmap.AlphaFormat := afDefined;
                BP := PBGRA32Array(FIL_FreeImage_GetBits(dib));
                LineNumber := 0;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].B := BP[LineNumber * PicBitmap.Width + x].B;
                        Line[x].G := BP[LineNumber * PicBitmap.Width + x].G;
                        Line[x].R := BP[LineNumber * PicBitmap.Width + x].R;
                        Line[x].A := BP[LineNumber * PicBitmap.Width + x].A;
                    end;
                    inc(LineNumber);
                end;
            finally
                PicBitmap.Canvas.UnLock;
            end;

        end else begin

            PicBitmap.Assign(nil);
            DC := GetDC(Handle);
            PicBitmap.Canvas.Lock;
            try
                PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                PicBitmap.PixelFormat := pf32bit;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].A := 255;
                    end;
                end;
            finally
                PicBitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromMemoryWithRescaleTBitmap(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    LoadDib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PBGRA32Array;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    MemoryHandle: PFIMEMORY;
    Line: PBGRA32Array;
    LineNumber: Integer;
    Width: Integer;
    Height: Integer;
    Perc: Single;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);

        //* Keep the aspect
        if KeepAspectRatio then begin

            if (NewWidth / Width) > (NewHeight / Height)
                then Perc := (NewHeight / Height)
                else Perc := (NewWidth / Width);

            //* If smaller don't resize
            if (Width < NewWidth)
            AND (Height < NewHeight)
            then begin
                Perc := 1;
            end;

            NewWidth := Round(Width * Perc);
            NewHeight := Round(Height * Perc);

        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, NewWidth, NewHeight, Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);
        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin

            PicBitmap.Canvas.Lock;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                PicBitmap.PixelFormat := pf32bit;
                PicBitmap.Transparent := True;
                //PicBitmap.AlphaFormat := afDefined;
                BP := PBGRA32Array(FIL_FreeImage_GetBits(dib));
                LineNumber := 0;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].B := BP[LineNumber * PicBitmap.Width + x].B;
                        Line[x].G := BP[LineNumber * PicBitmap.Width + x].G;
                        Line[x].R := BP[LineNumber * PicBitmap.Width + x].R;
                        Line[x].A := BP[LineNumber * PicBitmap.Width + x].A;
                    end;
                    inc(LineNumber);
                end;
            finally
                PicBitmap.Canvas.UnLock;
            end;
        end else begin
            PicBitmap.Assign(nil);
            DC := GetDC(Handle);
            PicBitmap.Canvas.Lock;
            try
                PicBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                PicBitmap.PixelFormat := pf32bit;
                for y := PicBitmap.Height - 1 downto 0 do begin
                    Line := PicBitmap.ScanLine[y];
                    for x := 0 to PicBitmap.Width - 1 do begin
                        Line[x].A := 255;
                    end;
                end;
            finally
                PicBitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;
{$ENDIF}

{$IFDEF USEGR32}
function FreeImageLoadImageTBitmap32(FileName: String; PicBitmap: TBitmap32; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; JPEGResizeSize: Integer = 0): Boolean;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    TAG: PFITAG;
    OriginalJPEGWidth, OriginalJPEGHeight: String;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        FileFormat := FreeImageGetFileFormat(PChar(FileName));
        if FileFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
            if JPEGResizeSize <> 0 then begin
               Flags := Flags OR (JPEGResizeSize SHL 16);
            end;
        end;
        try
            dib := FIL_FreeImage_Load(FileFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if (FileFormat = FIF_JPEG)
        AND (JPEGResizeSize <> 0)
        then begin
            // the dib may have been downscaled by 2x, 4x or 8x retrieve the original width & height (stored as comments for this special case)
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGWidth'), TAG);
            OriginalJPEGWidth := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGHeight'), TAG);
            OriginalJPEGHeight := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            OriginalWidth := StrToIntDef(OriginalJPEGWidth, - 1);
            OriginalHeight := StrToIntDef(OriginalJPEGHeight, - 1);
            if OriginalWidth = - 1 then begin
                OriginalWidth := FIL_FreeImage_GetWidth(dib);
            end;
            if OriginalHeight = - 1 then begin
                OriginalHeight := FIL_FreeImage_GetHeight(dib);
            end;
        end else begin
            OriginalWidth := FIL_FreeImage_GetWidth(dib);
            OriginalHeight := FIL_FreeImage_GetHeight(dib);
        end;

        if BPP = 32 then begin
            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;
        end else begin
            Bitmap := TBitmap.Create;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                Bitmap.Canvas.Lock;
                PicBitmap.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    if (Bitmap.Handle <> 0)
                    AND (Bitmap.Width > 0)
                    AND (Bitmap.Height > 0)
                    then begin
                        PicBitmap.Assign(Bitmap);
                    end;
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    PicBitmap.UnLock;
                    Bitmap.Canvas.UnLock;
                end;
            finally
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageTBitmap32BPP(FileName: String; PicBitmap: TBitmap32; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BitDepth: Integer; JPEGResizeSize: Integer = 0): Boolean;
var
    dib: PFIBITMAP;
    Tmpdib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    TAG: PFITAG;
    OriginalJPEGWidth, OriginalJPEGHeight: String;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        FileFormat := FreeImageGetFileFormat(PChar(FileName));
        if FileFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
            if JPEGResizeSize <> 0 then begin
               Flags := Flags OR (JPEGResizeSize SHL 16);
            end;
        end;
        try
            dib := FIL_FreeImage_Load(FileFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        BitDepth := BPP;

        if (FileFormat = FIF_JPEG)
        AND (JPEGResizeSize <> 0)
        then begin
            // the dib may have been downscaled by 2x, 4x or 8x retrieve the original width & height (stored as comments for this special case)
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGWidth'), TAG);
            OriginalJPEGWidth := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGHeight'), TAG);
            OriginalJPEGHeight := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            OriginalWidth := StrToIntDef(OriginalJPEGWidth, - 1);
            OriginalHeight := StrToIntDef(OriginalJPEGHeight, - 1);
        end else begin
            OriginalWidth := FIL_FreeImage_GetWidth(dib);
            OriginalHeight := FIL_FreeImage_GetHeight(dib);
        end;

        if BPP > 32 then begin
            Tmpdib := FIL_FreeImage_ConvertTo32Bits(dib);
            FIL_FreeImage_Unload(dib);
            dib := Tmpdib;
        end;

        if BPP >= 32 then begin

            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;

        end else begin
            Bitmap := TBitmap.Create;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);

                PicBitmap.Lock;
                Bitmap.Canvas.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    if (Bitmap.Handle <> 0)
                    AND (Bitmap.Width > 0)
                    AND (Bitmap.Height > 0)
                    then begin
                        PicBitmap.Assign(Bitmap);
                    end;
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    Bitmap.Canvas.UnLock;
                    PicBitmap.UnLock;
                end;

            finally
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageWithRescale(FileName: String; PicBitmap: TBitmap32; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    Width: Integer;
    Height: Integer;
    Perc: Double;
    LoadDib: PFIBITMAP;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        Perc := 1;
        DetectedFormat := FreeImageGetFileFormat(PChar(FileName));
        if DetectedFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_Load(DetectedFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);

        OriginalWidth := Width;
        OriginalHeight := Height;

        if KeepAspectRatio then begin
            //* Keep the aspect
            if (NewWidth / Width) > (NewHeight / Height)
                then Perc := (NewHeight / Height)
                else Perc := (NewWidth / Width);

            //* If smaller don't resize
            if (Width < NewWidth)
            AND (Height < NewHeight)
            then begin
                Perc := 1;
            end;
        end else begin
            //* Rescale
            Width := NewWidth;
            Height := NewHeight;
        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, Trunc(Perc * Width), Trunc(Perc * Height), Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;
        end else begin
            Bitmap := TBitmap.Create;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                Bitmap.Canvas.Lock;
                PicBitmap.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    if (Bitmap.Handle <> 0)
                    AND (Bitmap.Width > 0)
                    AND (Bitmap.Height > 0)
                    then begin
                        PicBitmap.Assign(Bitmap);
                    end;
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    PicBitmap.UnLock;
                    Bitmap.Canvas.UnLock;
                end;
            finally
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromHandleWithRescale(BitmapHandle: HBitmap; var PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    Width: Integer;
    Height: Integer;
    Perc: Double;
    LoadDib: PFIBITMAP;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    TempBitmap: TBitmap;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(BitmapHandle, sizeof(BITMAP), @bm);
    dib := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib);
    dc := GetDC(Handle);
    pdib := dib;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, BitmapHandle, 0, FIL_FreeImage_GetHeight(dib), FIL_FreeImage_GetBits(dib), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(Handle, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib).biClrImportant := nColors;
    try
        if Dib.data = nil
            then Exit;
        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);
        OriginalWidth := Width;
        OriginalHeight := Height;
        //* Keep the aspect
        if (NewWidth / Width) > (NewHeight / Height)
            then Perc := (NewHeight / Height)
            else Perc := (NewWidth / Width);
        //* If smaller don't resize
        //if (Width < NewWidth)
        //AND (Height < NewHeight)
        //then begin
        //    Perc := 1;
        //end;

        LoadDib := dib.data;

        dib := FIL_FreeImage_Rescale(dib, Trunc(Perc * Width), Trunc(Perc * Height), Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
            end;
        end else begin
            TempBitmap := TBitmap.Create;
            try
                TempBitmap.Assign(nil);
                DC := GetDC(Handle);
                TempBitmap.Canvas.Lock;
                TempBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                TempBitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                PicBitmap.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    PicBitmap.Assign(TempBitmap);
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    PicBitmap.UnLock;
                end;
            finally
                FreeAndNil(TempBitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib);
end;

function FreeImageLoadImageFromFreeImageWithRescale(FIBitmapHandle: PFIBITMAP; var PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    TempBitmap: TBitmap;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;

    try

        if FIBitmapHandle = nil
            then Exit;

        dib := FIL_FreeImage_Rescale(FIBitmapHandle, NewWidth, NewHeight, Resampler);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
            end;
        end else begin
            TempBitmap := TBitmap.Create;
            try
                TempBitmap.Assign(nil);
                DC := GetDC(Handle);
                TempBitmap.Canvas.Lock;
                TempBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                TempBitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                PicBitmap.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    PicBitmap.Assign(TempBitmap);
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    PicBitmap.UnLock;
                end;
            finally
                FreeAndNil(TempBitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;
{$ENDIF}

{$IFDEF USEGR32}
function FreeImageLoadImageFromMemoryTBitmap32(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; Handle: THandle; JPEGResizeSize: Integer = 0): Boolean;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    MemoryHandle: PFIMEMORY;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
            if JPEGResizeSize <> 0 then begin
               Flags := Flags OR (JPEGResizeSize SHL 16);
            end;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;
        end else begin
            Bitmap := TBitmap.Create;
            Bitmap.Canvas.Lock;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                if (Bitmap.Handle <> 0)
                AND (Bitmap.Width > 0)
                AND (Bitmap.Height > 0)
                then begin
                    PicBitmap.Lock;
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.Lock;
                    end;
                    try
                        PicBitmap.Assign(Bitmap);
                    finally
                        if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                            PicBitmap.Canvas.Unlock;
                        end;
                        PicBitmap.UnLock;
                    end;
                end;
            finally
                Bitmap.Canvas.UnLock;
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromMemoryTBitmap32WithRescale(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;
var
    dib, LoadDib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    MemoryHandle: PFIMEMORY;
    ActualWidth, ActualHeight: Integer;
    Perc: Double;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        ActualWidth := FIL_FreeImage_GetWidth(dib);
        ActualHeight := FIL_FreeImage_GetHeight(dib);

        if KeepAspectRatio then begin
            if (NewWidth / ActualWidth) > (NewHeight / ActualHeight)
                then Perc := (NewHeight / ActualHeight)
                else Perc := (NewWidth / ActualWidth);
            //* If smaller don't resize
            //if (Width < NewWidth)
            //AND (Height < NewHeight)
            //then begin
            //    Perc := 1;
            //end;
            ActualWidth := Trunc(Perc * ActualWidth);
            ActualHeight := Trunc(Perc * ActualHeight);
        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, ActualWidth, ActualHeight, Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;
        end else begin
            Bitmap := TBitmap.Create;
            Bitmap.Canvas.Lock;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                if (Bitmap.Handle <> 0)
                AND (Bitmap.Width > 0)
                AND (Bitmap.Height > 0)
                then begin
                    PicBitmap.Lock;
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.Lock;
                    end;
                    try
                        PicBitmap.Assign(Bitmap);
                    finally
                        if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                            PicBitmap.Canvas.Unlock;
                        end;
                        PicBitmap.UnLock;
                    end;
                end;
            finally
                Bitmap.Canvas.UnLock;
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromMemoryTBitmap32WithRescaleAndInfo(Location: Pointer; MemorySize: Int64; PicBitmap: TBitmap32; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle; var OriginalWidth: Integer; var OriginalHeight: Integer; var DetectedFormat: FREE_IMAGE_FORMAT; var BitDepth: Cardinal): Boolean;
var
    dib, LoadDib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    MemoryHandle: PFIMEMORY;
    ActualWidth, ActualHeight: Integer;
    Perc: Double;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        ActualWidth := FIL_FreeImage_GetWidth(dib);
        ActualHeight := FIL_FreeImage_GetHeight(dib);

        OriginalWidth := ActualWidth;
        OriginalHeight := ActualHeight;
        DetectedFormat := t;
        BitDepth := FIL_FreeImage_GetBPP(dib);

        if KeepAspectRatio then begin
            if (NewWidth / ActualWidth) > (NewHeight / ActualHeight)
                then Perc := (NewHeight / ActualHeight)
                else Perc := (NewWidth / ActualWidth);
            //* If smaller don't resize
            //if (Width < NewWidth)
            //AND (Height < NewHeight)
            //then begin
            //    Perc := 1;
            //end;
            ActualWidth := Trunc(Perc * ActualWidth);
            ActualHeight := Trunc(Perc * ActualHeight);
        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, ActualWidth, ActualHeight, Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin
            PicBitmap.Lock;
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
                PicBitmap.UnLock;
            end;
        end else begin
            Bitmap := TBitmap.Create;
            Bitmap.Canvas.Lock;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);
                if (Bitmap.Handle <> 0)
                AND (Bitmap.Width > 0)
                AND (Bitmap.Height > 0)
                then begin
                    PicBitmap.Lock;
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.Lock;
                    end;
                    try
                        PicBitmap.Assign(Bitmap);
                    finally
                        if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                            PicBitmap.Canvas.Unlock;
                        end;
                        PicBitmap.Unlock;
                    end;
                end;
            finally
                Bitmap.Canvas.UnLock;
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageAdjustGamma(Picture: HBitmap; Bitmap32: TBitmap32; Gamma: Double): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    x, y: integer;
    BP: PLONGWORD;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(Picture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;
    //* Adjust the gamme
    FIL_FreeImage_AdjustGamma(pdib, Gamma);
    //* Copy to output
    if Supports(Bitmap32.Backend, ICanvasSupport) then begin
        Bitmap32.Canvas.Lock;
    end;
    //Bitmap32.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
    BP := PLONGWORD(FIL_FreeImage_GetBits(pdib));
    for y := Bitmap32.Height - 1 downto 0 do begin
        for x := 0 to Bitmap32.Width - 1 do begin
            Bitmap32.Pixel[x, y] := BP^;
            inc(BP);
        end;
    end;
    if Supports(Bitmap32.Backend, ICanvasSupport) then begin
        Bitmap32.Canvas.UnLock;
    end;
    //* Clean up
    FIL_FreeImage_Unload(pdib);
end;

function FreeImageLoadImageWithRescaleIR(FileName: String; PicBitmap: TBitmap32; Handle: THandle; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; var OriginalWidth: Integer; var OriginalHeight: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT): Boolean; overload;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    Flags: Integer;
    Bitmap: TBitmap;
    Width: Integer;
    Height: Integer;
    Perc: Double;
    LoadDib: PFIBITMAP;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        DetectedFormat := FreeImageGetFileFormat(PChar(FileName));
        if DetectedFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_Load(DetectedFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        //* Rescale
        Width := FIL_FreeImage_GetWidth(dib);
        Height := FIL_FreeImage_GetHeight(dib);

        OriginalWidth := Width;
        OriginalHeight := Height;

        if KeepAspectRatio then begin
            //* Keep the aspect
            if (NewWidth / Width) > (NewHeight / Height)
                then Perc := (NewHeight / Height)
                else Perc := (NewWidth / Width);

            //* If smaller don't resize
            if (Width < NewWidth)
            AND (Height < NewHeight)
            then begin
                Perc := 1;
            end;

            NewWidth := Round(Width * Perc);
            NewHeight := Round(Height * Perc);

        end;

        LoadDib := dib;

        dib := FIL_FreeImage_Rescale(dib, NewWidth, NewHeight, Resampler);

        FIL_FreeImage_Unload(LoadDib);

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP = 32 then begin

            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
            BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
            for y := PicBitmap.Height - 1 downto 0 do begin
                for x := 0 to PicBitmap.Width - 1 do begin
                    PicBitmap.Pixel[x, y] := BP^;
                    inc(BP);
                end;
            end;
                               if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.UnLock;
            end;

        end else begin
            Bitmap := TBitmap.Create;
            try
                Bitmap.Assign(nil);
                DC := GetDC(Handle);
                Bitmap.Canvas.Lock;
                Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                Bitmap.Canvas.UnLock;
                ReleaseDC(Handle, DC);

                Bitmap.Canvas.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    if (Bitmap.Handle <> 0)
                    AND (Bitmap.Width > 0)
                    AND (Bitmap.Height > 0)
                    then begin
                        PicBitmap.Assign(Bitmap);
                    end;
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    Bitmap.Canvas.UnLock;
                end;

            finally
                FreeAndNil(Bitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromHandle(BitmapHandle: HBitmap; var PicBitmap: TBitmap32; Handle: THandle = 0): Boolean;
var
    dib: PFIBITMAP;
    pdib: PFIBITMAP;
    bm: tagBITMAP;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    x, y: integer;
    BP: PLONGWORD;
    DC: HDC;
    BPP: Longword;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    TempBitmap: TBitmap;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(BitmapHandle, sizeof(tagBITMAP), @bm);
    dib := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib);
    dc := GetDC(Handle);
    pdib := dib;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, BitmapHandle, 0, FIL_FreeImage_GetHeight(dib), FIL_FreeImage_GetBits(dib), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(Handle, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib).biClrImportant := nColors;
    try
        if Dib.data = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);
        BPP := FIL_FreeImage_GetBPP(dib);
        if BPP = 32 then begin
            if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                PicBitmap.Canvas.Lock;
            end;
            try
                PicBitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));
                BP := PLONGWORD(FIL_FreeImage_GetBits(dib));
                for y := PicBitmap.Height - 1 downto 0 do begin
                    for x := 0 to PicBitmap.Width - 1 do begin
                        PicBitmap.Pixel[x, y] := BP^;
                        inc(BP);
                    end;
                end;
            finally
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.UnLock;
                end;
            end;
        end else begin
            TempBitmap := TBitmap.Create;
            try
                TempBitmap.Assign(nil);
                DC := GetDC(Handle);
                TempBitmap.Canvas.Lock;
                try
                    TempBitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
                finally
                    TempBitmap.Canvas.UnLock;
                end;
                ReleaseDC(Handle, DC);
                PicBitmap.Lock;
                if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                    PicBitmap.Canvas.Lock;
                end;
                try
                    PicBitmap.Assign(TempBitmap);
                finally
                    if Supports(PicBitmap.Backend, ICanvasSupport) then begin
                        PicBitmap.Canvas.UnLock;
                    end;
                    PicBitmap.UnLock;
                end;
            finally
                FreeAndNil(TempBitmap);
            end;
        end;
        FIL_FreeImage_Unload(dib);
        if (PicBitmap.Width > 0)
        AND (PicBitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

{$ENDIF}

function FreeImageSaveImageToMemory(Picture: HBitmap; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    MemoryHandle: PFIMEMORY;
    PData: PByte;
    DataSize: Cardinal;
begin
    Result := False;
    if NOT FreeImageDLLLoaded then begin
        Exit;
    end;
    if Picture = 0 then begin
        Exit;
    end;
    GetObject(Picture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;
    MemoryHandle := FIL_FreeImage_OpenMemory;
    FIL_FreeImage_SaveToMemory(Format, dib.data, MemoryHandle, 0);
    FIL_FreeImage_AcquireMemory(MemoryHandle, PData, DataSize);
    MemoryStream.Clear;
    MemoryStream.Write(PData^, DataSize);
    FIL_FreeImage_CloseMemory(MemoryHandle);
    //* Save
    //Result := FIL_FreeImage_Save(Format, dib.data, PWideChar(FileName), Flags);
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib.data);
end;

function FreeImageLoadImage(FileName: String; var Bitmap: TBitmap; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BPP: longword; Handle: THandle): Boolean;
var
    dib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    x, y: integer;
    BP: PBGRA32Array;
    DC: HDC;
    Flags: Integer;
    Line: PBGRA32Array;
    LineNumber: Integer;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        FileFormat := FreeImageGetFileFormat(PChar(FileName));
        if FileFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;
        dib := FIL_FreeImage_Load(FileFormat, PChar(FileName), Flags);
        if Dib = nil
            then Exit;
        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        BPP := FIL_FreeImage_GetBPP(dib);
        OriginalWidth := FIL_FreeImage_GetWidth(dib);
        OriginalHeight := FIL_FreeImage_GetHeight(dib);

        //FreeImage_SetTransparent(dib,TRUE);

        if BPP = 32 then begin

            if NOT Assigned(Bitmap) then begin
                Bitmap := TBitmap.Create;
            end;

            Bitmap.Canvas.Lock;

            Bitmap.SetSize(FIL_FreeImage_GetWidth(dib), FIL_FreeImage_GetHeight(dib));

            Bitmap.PixelFormat := pf32bit;

            Bitmap.Transparent := True;

            BP := PBGRA32Array(FIL_FreeImage_GetBits(dib));
            LineNumber := 0;
            for y := Bitmap.Height - 1 downto 0 do begin
                Line := Bitmap.ScanLine[y];
                for x := 0 to Bitmap.Width - 1 do begin
                    Line[x].B := BP[LineNumber * Bitmap.Width + x].B;
                    Line[x].G := BP[LineNumber * Bitmap.Width + x].G;
                    Line[x].R := BP[LineNumber * Bitmap.Width + x].R;
                    Line[x].A := BP[LineNumber * Bitmap.Width + x].A;
                end;
                inc(LineNumber);
            end;

            Bitmap.Canvas.UnLock;

        end else begin

            if NOT Assigned(Bitmap) then begin
                Bitmap := TBitmap.Create;
            end;
            Bitmap.Assign(nil);
            DC := GetDC(Handle);
            Bitmap.Canvas.Lock;
            Bitmap.Handle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);

            Bitmap.PixelFormat := pf32bit;

            for y := Bitmap.Height - 1 downto 0 do begin
                Line := Bitmap.ScanLine[y];
                for x := 0 to Bitmap.Width - 1 do begin
                    Line[x].A := 255;
                end;
            end;

            Bitmap.Canvas.UnLock;
            ReleaseDC(Handle, DC);
        end;
        FIL_FreeImage_Unload(dib);
        if (Bitmap.Width > 0)
        AND (Bitmap.Height > 0)
        then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageResizeSaveImageToMemory(Picture: HBitmap; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean; var ResizedWidth: Integer; var ResizedHeight: Integer): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    MemoryHandle: PFIMEMORY;
    PData: PByte;
    DataSize: Cardinal;
    pdibResized: PFIBITMAP;
    OldWidth: Integer;
    OldHeight: Integer;
    Perc: Single;
begin
    Result := False;
    if NOT FreeImageDLLLoaded then begin
        Exit;
    end;
    if Picture = 0 then begin
        Exit;
    end;
    GetObject(Picture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;

    //* Rescale
    OldWidth := FIL_FreeImage_GetWidth(dib.data);
    OldHeight := FIL_FreeImage_GetHeight(dib.data);
    if KeepAspectRatio then begin
        //* Keep the aspect
        if (Width / OldWidth) > (Height / OldHeight)
            then Perc := (Height / OldHeight)
            else Perc := (Width / OldWidth);
        //* If smaller don't resize
        if (OldWidth < Width)
        AND (OldHeight < Height)
        then begin
            Perc := 1;
        end;

        Width := Round(OldWidth * Perc);
        Height := Round(OldHeight * Perc);
    end;

    ResizedWidth := Width;
    ResizedHeight := Height;

    pdibResized := FIL_FreeImage_Rescale(dib.data, Width, Height, FILTER_LANCZOS3);

    MemoryHandle := FIL_FreeImage_OpenMemory;
    FIL_FreeImage_SaveToMemory(Format, pdibResized, MemoryHandle, 0);

    FIL_FreeImage_AcquireMemory(MemoryHandle, PData, DataSize);

    MemoryStream.Clear;
    MemoryStream.Write(PData^, DataSize);
    FIL_FreeImage_CloseMemory(MemoryHandle);
    //* Save
    //Result := FIL_FreeImage_Save(Format, dib.data, PWideChar(FileName), Flags);
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib.data);
    FIL_FreeImage_Unload(pdibResized);
end;

function FreeImageLoadResizeSaveImageToMemory(FileName: String; SaveFormat: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean; var DetectedFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var ResizedWidth: Integer; var ResizedHeight: Integer; var BitDepth: Cardinal): Boolean;
var
    dib: PFIBITMAP;
    LoadFlags: Integer;
    MemoryHandle: PFIMEMORY;
    PData: PByte;
    DataSize: Cardinal;
    pdibResized: PFIBITMAP;
    Perc: Single;
    SaveFlags: Integer;
begin
    Result := False;
    if NOT FreeImageDLLLoaded then begin
        Exit;
    end;
    if NOT FileExists(FileName) then begin
        Exit;
    end;

    LoadFlags := 0;
    DetectedFormat := FreeImageGetFileFormat(PChar(FileName));
    if DetectedFormat = FIF_JPEG then begin
        LoadFlags := JPEG_ACCURATE;
    end;
    dib := FIL_FreeImage_Load(DetectedFormat, PChar(FileName), LoadFlags);
    if Dib = nil
        then Exit;

    //* Rescale
    OriginalWidth := FIL_FreeImage_GetWidth(dib);
    OriginalHeight := FIL_FreeImage_GetHeight(dib);
    if KeepAspectRatio then begin
        //* Keep the aspect
        if (Width / OriginalWidth) > (Height / OriginalHeight)
            then Perc := (Height / OriginalHeight)
            else Perc := (Width / OriginalWidth);
        //* If smaller don't resize
        if (OriginalWidth < Width)
        AND (OriginalHeight < Height)
        then begin
            Perc := 1;
        end;

        Width := Round(OriginalWidth * Perc);
        Height := Round(OriginalHeight * Perc);
    end;

    ResizedWidth := Width;
    ResizedHeight := Height;

    pdibResized := FIL_FreeImage_Rescale(dib, Width, Height, FILTER_LANCZOS3);

    SaveFlags := 0;
    if SaveFormat = FIF_JPEG then begin
        SaveFlags := JPEG_QUALITYSUPERB OR JPEG_SUBSAMPLING_444 OR JPEG_OPTIMIZE;
    end;

    MemoryHandle := FIL_FreeImage_OpenMemory;
    FIL_FreeImage_SaveToMemory(SaveFormat, pdibResized, MemoryHandle, SaveFlags);

    BitDepth := FIL_FreeImage_GetBPP(pdibResized);

    FIL_FreeImage_AcquireMemory(MemoryHandle, PData, DataSize);

    MemoryStream.Clear;
    MemoryStream.Write(PData^, DataSize);
    FIL_FreeImage_CloseMemory(MemoryHandle);
    //* Save
    //Result := FIL_FreeImage_Save(Format, dib.data, PWideChar(FileName), Flags);
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib);
    FIL_FreeImage_Unload(pdibResized);
    Result := True;
end;

function FreeImageLoadFromMemoryResizeSaveImageToMemory(Location: Pointer; MemorySize: Int64; Format: FREE_IMAGE_FORMAT; Flags: Integer; MemoryStream: TMemoryStream; Width: Integer; Height: Integer; KeepAspectRatio: Boolean): Boolean;
var
    dib: PFIBITMAP;
    t: FREE_IMAGE_FORMAT;
    LoadFlags: Integer;
    MemoryHandle: PFIMEMORY;
    LoadMemoryHandle: PFIMEMORY;
    PData: PByte;
    DataSize: Cardinal;
    pdibResized: PFIBITMAP;
    OldWidth: Integer;
    OldHeight: Integer;
    Perc: Single;
begin
    Result := False;
    if NOT FreeImageDLLLoaded then begin
        Exit;
    end;
    if NOT Assigned(Location)
    OR (MemorySize = 0)
    then begin
        Exit;
    end;
    LoadFlags := 0;
    LoadMemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
    t := FIL_FreeImage_GetFileTypeFromMemory(LoadMemoryHandle, 0);
    try
        if t = FIF_JPEG then begin
            LoadFlags := JPEG_ACCURATE;
        end;
        try
            dib := FIL_FreeImage_LoadFromMemory(t, LoadMemoryHandle, LoadFlags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        //* Rescale
        OldWidth := FIL_FreeImage_GetWidth(dib);
        OldHeight := FIL_FreeImage_GetHeight(dib);
        if KeepAspectRatio then begin
            //* Keep the aspect
            if (Width / OldWidth) > (Height / OldHeight)
                then Perc := (Height / OldHeight)
                else Perc := (Width / OldWidth);
            //* If smaller don't resize
            if (OldWidth < Width)
            AND (OldHeight < Height)
            then begin
                Perc := 1;
            end;

            Width := Round(OldWidth * Perc);
            Height := Round(OldHeight * Perc);
        end;

        pdibResized := FIL_FreeImage_Rescale(dib, Width, Height, FILTER_LANCZOS3);

        MemoryHandle := FIL_FreeImage_OpenMemory;
        FIL_FreeImage_SaveToMemory(Format, pdibResized, MemoryHandle, 0);

        FIL_FreeImage_AcquireMemory(MemoryHandle, PData, DataSize);

        MemoryStream.Clear;
        MemoryStream.Write(PData^, DataSize);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        //* Save
        //Result := FIL_FreeImage_Save(Format, dib.data, PWideChar(FileName), Flags);
        // don't forget to call FreeImage_Unload when you no longer need the dib
        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_Unload(pdibResized);
        Result := True;
    finally
        FIL_FreeImage_CloseMemory(LoadMemoryHandle);
    end;
end;

function FreeImageSaveImageQuantize(Picture: HBitmap; FileName: String; Format: FREE_IMAGE_FORMAT; Flags: Integer): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;

    pdib24: PFIBITMAP;
    pdibQuantized: PFIBITMAP;

begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(Picture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, Picture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;
    //* Convert to 24 bit
    pdib24 := FIL_FreeImage_ConvertTo24Bits(pdib);
    FIL_FreeImage_Unload(pdib);
    ///* Quantize
    pdibQuantized := FIL_FreeImage_ColorQuantize(pdib24, FIQ_NNQUANT);
    FIL_FreeImage_Unload(pdib24);
    //* Save
    Result := FIL_FreeImage_Save(Format, pdibQuantized, PWideChar(FileName), Flags);
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(pdibQuantized);
end;

function FreeImageGetJPEGDimensions(FileName: String; var OriginalWidth: Integer; var OriginalHeight: Integer): Boolean;
var
    dib: PFIBITMAP;
    LoadFlags: Integer;
    OriginalJPEGWidth, OriginalJPEGHeight: String;
    TAG: PFITAG;
begin
    Result := False;
    if NOT FreeImageDLLLoaded then begin
        Exit;
    end;
    LoadFlags := 0 OR (32 SHL 16);
    dib := FIL_FreeImage_Load(FIF_JPEG, PChar(FileName), LoadFlags);
    if Dib = nil
        then Exit;
    // the dib may have been downscaled by 2x, 4x or 8x retrieve the original width & height (stored as comments for this special case)
    FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGWidth'), TAG);
    OriginalJPEGWidth := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
    FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGHeight'), TAG);
    OriginalJPEGHeight := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
    OriginalWidth := StrToIntDef(OriginalJPEGWidth, - 1);
    OriginalHeight := StrToIntDef(OriginalJPEGHeight, - 1);
    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(dib);
    Result := True;
end;

function FreeImageGetImageAttributes(FileName: String; Handle: THandle; var FileFormat: FREE_IMAGE_FORMAT; var OriginalWidth: Integer; var OriginalHeight: Integer; var BitDepth: Cardinal; JPEGResizeSize: Integer = 0): Boolean;
var
    dib: PFIBITMAP;
    BPP: Longword;
    Flags: Integer;
    TAG: PFITAG;
    OriginalJPEGWidth, OriginalJPEGHeight: String;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;
        FileFormat := FreeImageGetFileFormat(PChar(FileName));
        if FileFormat = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
            if JPEGResizeSize <> 0 then begin
               Flags := Flags OR (JPEGResizeSize SHL 16);
            end;
        end;
        try
            dib := FIL_FreeImage_Load(FileFormat, PChar(FileName), Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        BPP := FIL_FreeImage_GetBPP(dib);
        BitDepth := BPP;

        if (FileFormat = FIF_JPEG)
        AND (JPEGResizeSize <> 0)
        then begin
            // the dib may have been downscaled by 2x, 4x or 8x retrieve the original width & height (stored as comments for this special case)
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGWidth'), TAG);
            OriginalJPEGWidth := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            FIL_FreeImage_GetMetadata(FIMD_COMMENTS, dib, PANSIChar('OriginalJPEGHeight'), TAG);
            OriginalJPEGHeight := String(PANSIChar(FIL_FreeImage_GetTagValue(tag)));
            OriginalWidth := StrToIntDef(OriginalJPEGWidth, - 1);
            OriginalHeight := StrToIntDef(OriginalJPEGHeight, - 1);
        end else begin
            OriginalWidth := FIL_FreeImage_GetWidth(dib);
            OriginalHeight := FIL_FreeImage_GetHeight(dib);
        end;

        FIL_FreeImage_Unload(dib);
        Result := True;
    except
        //*
    end;
end;

function FreeImageLoadImageFromMemoryBitmapHandleWithRescale(Location: Pointer; MemorySize: Int64; var BitmapHandle: HBitmap; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; Handle: THandle): Boolean;
var
    dib, LoadDib: PFIBITMAP;
    PBH: PBITMAPINFOHEADER;
    PBI: PBITMAPINFO;
    t: FREE_IMAGE_FORMAT;
    DC: HDC;
    Flags: Integer;
    MemoryHandle: PFIMEMORY;
    ActualWidth, ActualHeight: Integer;
    Perc: Double;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        if t = FIF_JPEG then begin
            Flags := JPEG_ACCURATE;
        end;

        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        if (NewWidth > 0)
        AND (NewHeight > 0)
        then begin

            ActualWidth := FIL_FreeImage_GetWidth(dib);
            ActualHeight := FIL_FreeImage_GetHeight(dib);

            if KeepAspectRatio then begin
                if (NewWidth / ActualWidth) > (NewHeight / ActualHeight)
                    then Perc := (NewHeight / ActualHeight)
                    else Perc := (NewWidth / ActualWidth);
                //* If smaller don't resize
                //if (Width < NewWidth)
                //AND (Height < NewHeight)
                //then begin
                //    Perc := 1;
                //end;
                ActualWidth := Trunc(Perc * ActualWidth);
                ActualHeight := Trunc(Perc * ActualHeight);
            end;

            LoadDib := dib;
            dib := FIL_FreeImage_Rescale(dib, ActualWidth, ActualHeight, Resampler);
            FIL_FreeImage_Unload(LoadDib);
        end;

        PBH := FIL_FreeImage_GetInfoHeader(dib);
        PBI := FIL_FreeImage_GetInfo(dib^);

        DC := GetDC(Handle);
        BitmapHandle := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(dib)), PBI^, DIB_RGB_COLORS);
        ReleaseDC(Handle, DC);

        FIL_FreeImage_Unload(dib);
        FIL_FreeImage_CloseMemory(MemoryHandle);
        if BitmapHandle <> 0 then begin
            Result := True;
        end;
    except
        //*
    end;
end;

function FreeImageLoadImageFromMemoryWithRescaleAndSave(Location: Pointer; MemorySize: Int64; FileName: String; NewWidth: Integer; NewHeight: Integer; Resampler: FREE_IMAGE_FILTER; KeepAspectRatio: Boolean; SaveFormat: FREE_IMAGE_FORMAT): Boolean;
var
    dib, LoadDib, pdib24, pdib8: PFIBITMAP;
    t: FREE_IMAGE_FORMAT;
    BPP: Longword;
    Flags: Integer;
    MemoryHandle: PFIMEMORY;
    ActualWidth, ActualHeight: Integer;
    Perc: Double;
begin
    Result := False;
    try
        if NOT FreeImageDLLLoaded
            then Exit;
        Flags := 0;

        MemoryHandle := FIL_FreeImage_OpenMemory(Location, MemorySize);
        t := FIL_FreeImage_GetFileTypeFromMemory(MemoryHandle, 0);

        try
            dib := FIL_FreeImage_LoadFromMemory(t, MemoryHandle, Flags);
        except
            dib := nil;
        end;
        if Dib = nil
            then Exit;

        BPP := FIL_FreeImage_GetBPP(dib);

        if BPP < 24 then begin
            //* Convert to 24 bit for better quality resize
            pdib24 := FIL_FreeImage_ConvertTo24Bits(dib);
            FIL_FreeImage_Unload(dib);
            dib := pdib24;
        end;

        if (NewWidth > 0)
        AND (NewHeight > 0)
        then begin

            ActualWidth := FIL_FreeImage_GetWidth(dib);
            ActualHeight := FIL_FreeImage_GetHeight(dib);

            if KeepAspectRatio then begin
                if (NewWidth / ActualWidth) > (NewHeight / ActualHeight)
                    then Perc := (NewHeight / ActualHeight)
                    else Perc := (NewWidth / ActualWidth);
                //* If smaller don't resize
                //if (Width < NewWidth)
                //AND (Height < NewHeight)
                //then begin
                //    Perc := 1;
                //end;
                ActualWidth := Trunc(Perc * ActualWidth);
                ActualHeight := Trunc(Perc * ActualHeight);
            end;

            LoadDib := dib;
            dib := FIL_FreeImage_Rescale(dib, ActualWidth, ActualHeight, Resampler);
            FIL_FreeImage_Unload(LoadDib);
        end;

        BPP := FIL_FreeImage_GetBPP(dib);

        if (BPP <> 24)
        AND (SaveFormat = FIF_JPEG)
        then begin
            //* Convert to 24 bit, JPEG requires 24 bit
            pdib24 := FIL_FreeImage_ConvertTo24Bits(dib);
            FIL_FreeImage_Unload(dib);
            dib := pdib24;
        end;
        if SaveFormat = FIF_JPEG then begin
            Flags := JPEG_QUALITYSUPERB;
        end;

        if (BPP <> 8)
        AND (SaveFormat = FIF_GIF)
        then begin
            //* Convert to 8 bit, GIF is at max. 8 bit
            pdib8 := FIL_FreeImage_ColorQuantize(dib, FIQ_NNQUANT);
            FIL_FreeImage_Unload(dib);
            dib := pdib8;
        end;

        //* Save
        Result := FIL_FreeImage_Save(SaveFormat, dib, PWideChar(FileName), Flags);

        // don't forget to call FreeImage_Unload when you no longer need the dib
        FIL_FreeImage_Unload(dib);

        FIL_FreeImage_CloseMemory(MemoryHandle);
        Result := True;

    except
        //*
    end;
end;

function FreeImageQuantize(InPicture: HBitmap; var OutPicture: HBitmap; Quantize: FREE_IMAGE_QUANTIZE = FIQ_NNQUANT): Boolean;
var
    dib: FIBITMAP;
    pdib: PFIBITMAP;
    bm: BITMAP;
    dc: HDC;
    BitmapInfo: TBitmapInfo;
    PointerBitmapInfo: PBitmapInfo;
    nColors: Integer;
    PBH: PBITMAPINFOHEADER;
    pdib24: PFIBITMAP;
    pdibQuantized: PFIBITMAP;
    PBI: PBITMAPINFO;
begin
    Result := False;
    if NOT FreeImageDLLLoaded
        then Exit;
    GetObject(InPicture, sizeof(BITMAP), @bm);
    dib.data := FIL_FreeImage_Allocate(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
    nColors := FIL_FreeImage_GetColorsUsed(dib.data);
    dc := GetDC(0);
    pdib := dib.data;
    PointerBitmapInfo := FIL_FreeImage_GetInfo(pdib^);
    BitmapInfo.bmiHeader := PointerBitmapInfo.bmiHeader;
    BitmapInfo.bmiColors := PointerBitmapInfo.bmiColors;
    GetDIBits(dc, InPicture, 0, FIL_FreeImage_GetHeight(dib.data), FIL_FreeImage_GetBits(dib.data), BitmapInfo, DIB_RGB_COLORS);
    ReleaseDC(0, dc);
    // restore BITMAPINFO members
    FIL_FreeImage_GetInfoHeader(dib.data).biClrUsed := nColors;
    FIL_FreeImage_GetInfoHeader(dib.data).biClrImportant := nColors;
    //* Convert to 24 bit
    pdib24 := FIL_FreeImage_ConvertTo24Bits(pdib);
    FIL_FreeImage_Unload(pdib);
    ///* Quantize
    pdibQuantized := FIL_FreeImage_ColorQuantize(pdib24, Quantize);
    FIL_FreeImage_Unload(pdib24);

    DC := GetDC(0);
    try
        PBH := FIL_FreeImage_GetInfoHeader(pdibQuantized);
        PBI := FIL_FreeImage_GetInfo(pdibQuantized^);
        OutPicture := CreateDIBitmap(DC, PBH^, CBM_INIT, PChar(FIL_FreeImage_GetBits(pdibQuantized)), PBI^, DIB_RGB_COLORS);
    finally
        ReleaseDC(0, DC);
    end;

    // don't forget to call FreeImage_Unload when you no longer need the dib
    FIL_FreeImage_Unload(pdibQuantized);
end;

Initialization

    InitFreeImageLibrary

Finalization

    FreeFreeImageLibrary;

end.
