//********************************************************************************************************************************
//*                                                                                                                              *
//*     Image Recognition Library  3delite 2011-2021                                                                            *
//*     See Image Recognition Library ReadMe.txt for details                                                                     *
//*                                                                                                                              *
//* Licenses available for usage of this library:                                                                                *
//* Freeware License: 25                                                                                                        *
//*     http://www.shareit.com/product.html?productid=300983850                                                                  *
//* Shareware License: 50                                                                                                       *
//*     http://www.shareit.com/product.html?productid=300486940                                                                  *
//* Commercial License: 250                                                                                                     *
//*     http://www.shareit.com/product.html?productid=300486939                                                                  *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/ImageRecognitionLibrary.html                              *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, ImageRecognitionDefs, Vcl.ExtCtrls, Vcl.ComCtrls, UITypes;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Button1: TButton;
    Label1: TLabel;
    Edit1: TEdit;
    Button2: TButton;
    Edit2: TEdit;
    Button3: TButton;
    Label2: TLabel;
    Memo1: TMemo;
    GroupBox2: TGroupBox;
    Image1: TImage;
    OpenDialog1: TOpenDialog;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Label3: TLabel;
    Label4: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Image2: TImage;
    ProgressBar1: TProgressBar;
    CheckBox1: TCheckBox;
    Label14: TLabel;
    TrackBar9: TTrackBar;
    Label13: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure TrackBar9Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
    STARTUP_MESSAGE = 'This tutorial demonstrates performing a search in "exact" mode. '
        + 'This requires that the search for bitmap is pixel identical with the search in bitmap. '
        + 'Although you can set a "DifferenceTolerance" value to allow a concrete pixel count to differ. '
        + 'Exact mode is the fastest of the available modes.'
        ;

var
    Form1: TForm1;
    IRObjectSearchIn: TIRObject;
    IRObjectSearchFor: TIRObject;
    IRCreateObjectParameters: TIRCreateObjectParameters;

implementation

{$R *.dfm}

Uses
    FreeImageLoader;

function mSec2Time(mSec: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - ((StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60) + StrToInt(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    //tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    if ImageRecognitionLibraryLoaded then begin
        Memo1.Lines.Add(ImageRecognitionLibraryName + ' loaded.');
        Memo1.Lines.Add(Caption + ' ready.');
    end else begin
        Memo1.Lines.Add('Error loading ' + ImageRecognitionLibraryName);
    end;
    TrackBar9Change(nil);
    MessageDlg(STARTUP_MESSAGE, mtInformation, [mbOk], 0);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
    Close;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
    if OpenDialog1.Execute then begin
        Edit1.Text := OpenDialog1.FileName;
    end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
    if OpenDialog1.Execute then begin
        Edit2.Text := OpenDialog1.FileName;
    end;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
    Error: Integer;
    Bitmap: TBitmap;
begin
    Bitmap := nil;
    //* Clear all
    ZeroMemory(@IRObjectSearchIn, SizeOf(TIRObject));
    //* Clear all
    ZeroMemory(@IRCreateObjectParameters, SizeOf(TIRCreateObjectParameters));
    //* No pre-processing of the image
    IRCreateObjectParameters.ProcessType := IR_PROCESS_TYPE_NONE;
    //* Create the object
    Error := ImageRecognition_CreateObject(PChar(Edit1.Text), IRObjectSearchIn, IRCreateObjectParameters);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Created object: $' + IntToHex(NativeInt(IRObjectSearchIn.ImageObject), 8) + ' ' + IntToStr(IRObjectSearchIn.OriginalDimensions.Width) + 'x' + IntToStr(IRObjectSearchIn.OriginalDimensions.Height));
        FreeImageLoadImage(Edit1.Text, Bitmap, Handle);
        Image1.Picture.Assign(Bitmap);
        FreeAndNil(Bitmap);
    end else begin
        Memo1.Lines.Add('Error creating object, error code: ' + IntToStr(Error));
    end;
    //* Clear all
    ZeroMemory(@IRObjectSearchFor, SizeOf(TIRObject));
    //* Clear all
    ZeroMemory(@IRCreateObjectParameters, SizeOf(TIRCreateObjectParameters));
    //* No pre-processing of the image
    IRCreateObjectParameters.ProcessType := IR_PROCESS_TYPE_NONE;
    //* Create the object
    Error := ImageRecognition_CreateObject(PChar(Edit2.Text), IRObjectSearchFor, IRCreateObjectParameters);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Created object: $' + IntToHex(NativeInt(IRObjectSearchFor.ImageObject), 8) + ' ' + IntToStr(IRObjectSearchFor.OriginalDimensions.Width) + 'x' + IntToStr(IRObjectSearchFor.OriginalDimensions.Height));
        FreeImageLoadImage(Edit2.Text, Bitmap, Handle);
        Image2.Picture.Assign(Bitmap);
        FreeAndNil(Bitmap);
    end else begin
        Memo1.Lines.Add('Error creating object, error code: ' + IntToStr(Error));
    end;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
    Error: Integer;
begin
    //* Search in object
    Error := ImageRecognition_FreeObject(IRObjectSearchIn);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Freed object: ' + IRObjectSearchIn.FileName);
        Image1.Picture.Assign(nil);
    end else begin
        Memo1.Lines.Add('Error freeing object, error code: ' + IntToStr(Error));
    end;
    //* Search for object
    Error := ImageRecognition_FreeObject(IRObjectSearchFor);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Freed object: ' + IRObjectSearchFor.FileName);
        Image2.Picture.Assign(nil);
    end else begin
        Memo1.Lines.Add('Error freeing object, error code: ' + IntToStr(Error));
    end;
end;

function StatusCallback(Progress: Double; StatusType: Integer; User: Pointer): Integer; {$IFDEF MSWINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
    Form1.ProgressBar1.Position := Round(Progress);
    //* Return IR_ABORT_PROCESSING to abort the processing
    Result := IR_CONTINUE_PROCESSING;
end;

procedure TForm1.Button6Click(Sender: TObject);
var
    Parameters: TIRProcessParameters;
    Results: TIRResult;
    Error: Integer;
    PerfFreq, FreqCountStart, FreqCountEnd: Int64;
    ProcessingTime: Integer;
    PMatch: PIRResultMatches;
    i: Integer;
begin
    ProgressBar1.Position := 0;

    //* Clear all
    ZeroMemory(@Parameters, SizeOf(TIRProcessParameters));

    //* We do an "exact" search
    Parameters.CompareType := IR_COMPARE_TYPE_EXACT;
    //* Allow multiple matches
    Parameters.MultipleMatchesMax := 255;
    //* Number of pixels that may differ completely
    Parameters.DifferenceTolerance := Trunc(IRObjectSearchFor.OriginalDimensions.Width * IRObjectSearchFor.OriginalDimensions.Height * (TrackBar9.Position / 100));
    //* A callback to get notified about the progress
    //* Note that this can significantly reduce performance
    Parameters.StatusCallback := StatusCallback;

    //* Filter out matches that are duplicates - their X-Y position difference is less than 10 pixels
    Parameters.MultipleMatchesFilterTresholdX := 10;
    Parameters.MultipleMatchesFilterTresholdY := 10;

    //* Use multi-threaded processing
    //* For an exact search with small < 1000x1000 pixels bitmap it is not recommended
    //* to use this feature because of the thread creation overhead
    if CheckBox1.Checked then begin
        Parameters.MultiThreadedProcessing := True;
    end else begin
        Parameters.MultiThreadedProcessing := False;
    end;

    QueryPerformanceFrequency(PerfFreq);
    QueryPerformanceCounter(FreqCountStart);

    //* Perform the search
    Error := ImageRecognition_Compare(IRObjectSearchIn, IRObjectSearchFor, Parameters, Results, nil);

    QueryPerformanceCounter(FreqCountEnd);
    ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);

    if Error = IR_OK then begin
        //* The processing completed normally
    end else begin
        Memo1.Lines.Add('Error while processing, error code: ' + IntToStr(Error));
    end;
    if Results.Success then begin
        Memo1.Lines.Add('Successful search, match count: ' + IntToStr(Results.MatchCount));
        //* Create a rectangle sorrounding the matching part of the picture
        Image1.Canvas.Pen.Color := clRed;
        Image1.Canvas.Pen.Width := 5;
        Image1.Canvas.Brush.Style := bsSolid;
        Image1.Canvas.Brush.Color := clRed;
        //* Point to the first match
        PMatch := Results.Matches;
        //* Iterate over the matches
        for i := 0 to Results.MatchCount - 1 do begin
            Memo1.Lines.Add('Found at position: '
                + IntToStr(PMatch.Position.X)
                + 'x'
                + IntToStr(PMatch.Position.Y)
                + ' difference: '
                + IntToStr(PMatch.Difference));
            Image1.Canvas.MoveTo(PMatch.Position.X, PMatch.Position.Y);
            Image1.Canvas.LineTo(PMatch.Position.X + PMatch.Dimensions.Width, PMatch.Position.Y);
            Image1.Canvas.LineTo(PMatch.Position.X + PMatch.Dimensions.Width, PMatch.Position.Y + PMatch.Dimensions.Height);
            Image1.Canvas.LineTo(PMatch.Position.X, PMatch.Position.Y + PMatch.Dimensions.Height);
            Image1.Canvas.LineTo(PMatch.Position.X, PMatch.Position.Y);
            //* Point to the next match
            Inc(PMatch);
        end;
        Memo1.Lines.Add('Processing time: ' + mSec2Time(ProcessingTime));
        Image1.Update;
        Image1.Invalidate;
        //* After finished with the result object free the allocated memory
        ImageRecognition_FreeResult(Results);
    end else begin
        Memo1.Lines.Add('No match found.');
    end;
end;

procedure TForm1.TrackBar9Change(Sender: TObject);
begin
    Label14.Caption := IntToStr(TrackBar9.Position) + ' %';
end;

end.
