//********************************************************************************************************************************
//*                                                                                                                              *
//*     Image Recognition Library  3delite 2011-2021                                                                            *
//*     See Image Recognition Library ReadMe.txt for details                                                                     *
//*                                                                                                                              *
//* Licenses available for usage of this library:                                                                                *
//* Freeware License: 25                                                                                                        *
//*     http://www.shareit.com/product.html?productid=300983850                                                                  *
//* Shareware License: 50                                                                                                       *
//*     http://www.shareit.com/product.html?productid=300486940                                                                  *
//* Commercial License: 250                                                                                                     *
//*     http://www.shareit.com/product.html?productid=300486939                                                                  *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object%20Pascal%20Developer%20Resources/ImageRecognitionLibrary.html                              *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, ImageRecognitionDefs, Vcl.ExtCtrls, Vcl.ComCtrls, UITypes, Vcl.Samples.Spin;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Button1: TButton;
    Label1: TLabel;
    Edit1: TEdit;
    Button2: TButton;
    Edit2: TEdit;
    Button3: TButton;
    Label2: TLabel;
    Memo1: TMemo;
    GroupBox2: TGroupBox;
    Image1: TImage;
    OpenDialog1: TOpenDialog;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Label3: TLabel;
    Label4: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Image2: TImage;
    ProgressBar1: TProgressBar;
    Label14: TLabel;
    TrackBar9: TTrackBar;
    Label13: TLabel;
    Label6: TLabel;
    ComboBoxOpenCLPlatform: TComboBox;
    ComboBoxOpenCLDevice: TComboBox;
    Label7: TLabel;
    Button7: TButton;
    Edit6: TEdit;
    Label8: TLabel;
    TrackBar4: TTrackBar;
    TrackBar3: TTrackBar;
    Edit5: TEdit;
    Label5: TLabel;
    Label9: TLabel;
    Edit4: TEdit;
    TrackBar2: TTrackBar;
    TrackBar1: TTrackBar;
    Edit3: TEdit;
    Label10: TLabel;
    ComboBoxCores: TComboBox;
    Label11: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure TrackBar9Change(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ComboBoxOpenCLPlatformSelect(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure TrackBar2Change(Sender: TObject);
    procedure TrackBar3Change(Sender: TObject);
    procedure TrackBar4Change(Sender: TObject);
    procedure Edit3Exit(Sender: TObject);
    procedure Edit4Exit(Sender: TObject);
    procedure Edit5Exit(Sender: TObject);
    procedure Edit6Exit(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ListOpenCLPlatforms;
    procedure ListOpenCLPlatformDevices(Index: Integer);
  end;

const
    STARTUP_MESSAGE = 'This tutorial demonstrates performing a search in relative ARGB mode with alpha channel with OpenCL. '
        + 'This means that the search for bitmap is compared with an allowed difference value and transparency of the search for picture. '
        + 'Setting a "DifferenceTolerance" value will allow a concrete pixel count to differ completely. '
        + 'Search for bitmap for alpha channel compare should be PNG with alpha channel transparency. '
        ;

var
    Form1: TForm1;
    IRObjectSearchIn: TIRObject;
    IRObjectSearchFor: TIRObject;
    IRCreateObjectParameters: TIRCreateObjectParameters;
    IROCLDevice: TIROCLDevice;

implementation

{$R *.dfm}

Uses
    FreeImageLoader;

function mSec2Time(mSec: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - ((StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60) + StrToInt(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    //tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    if ImageRecognitionLibraryLoaded then begin
        Memo1.Lines.Add(ImageRecognitionLibraryName + ' loaded.');
        Memo1.Lines.Add(Caption + ' ready.');
    end else begin
        Memo1.Lines.Add('Error loading ' + ImageRecognitionLibraryName);
    end;
    TrackBar1Change(nil);
    TrackBar2Change(nil);
    TrackBar3Change(nil);
    TrackBar4Change(nil);
    TrackBar9Change(nil);
    MessageDlg(STARTUP_MESSAGE, mtInformation, [mbOk], 0);
    ListOpenCLPlatforms;
    if ComboBoxOpenCLPlatform.Items.Count > 0 then begin
        ComboBoxOpenCLPlatform.ItemIndex := 0;
    end;
    ListOpenCLPlatformDevices(0);
    if ComboBoxOpenCLDevice.Items.Count > 0 then begin
        ComboBoxOpenCLDevice.ItemIndex := 0;
    end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
    if Assigned(IROCLDevice.Instance) then begin
        ImageRecognition_OpenCLFree(IROCLDevice);
    end;
end;

procedure TForm1.ListOpenCLPlatforms;
var
    i: Integer;
    PlatformName: PChar;
begin
    ComboBoxOpenCLPlatform.Clear;
    i := - 1;
    repeat
        Inc(i);
        PlatformName := ImageRecognition_OpenCLGetPlatform(i);
        if NOT Assigned(PlatformName) then begin
            Break;
        end;
        ComboBoxOpenCLPlatform.Items.Append(PlatformName);
    until (False);
    if ComboBoxOpenCLPlatform.Items.Count = 0 then begin
        ComboBoxOpenCLPlatform.Items.Append('No platform detected');
    end;
end;

procedure TForm1.ListOpenCLPlatformDevices(Index: Integer);
var
    i: Integer;
    DeviceName: PChar;
begin
    ComboBoxOpenCLDevice.Clear;
    i := - 1;
    repeat
        Inc(i);
        DeviceName := ImageRecognition_OpenCLGetPlatformDevice(Index, i);
        if NOT Assigned(DeviceName) then begin
            Break;
        end;
        ComboBoxOpenCLDevice.Items.Append(DeviceName);
    until (False);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
    Close;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
    if OpenDialog1.Execute then begin
        Edit1.Text := OpenDialog1.FileName;
    end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
    if OpenDialog1.Execute then begin
        Edit2.Text := OpenDialog1.FileName;
    end;
end;

procedure TForm1.Button4Click(Sender: TObject);
var
    Error: Integer;
    Bitmap: TBitmap;
    PerfFreq, FreqCountStart, FreqCountEnd: Int64;
    ProcessingTime: Integer;
begin
    if NOT Assigned(IROCLDevice.Instance) then begin
        MessageDlg('First please initialize the OpenCL device.', mtError, [mbCancel], 0);
        Exit;
    end;
    Bitmap := nil;
    //* Clear all
    ZeroMemory(@IRObjectSearchIn, SizeOf(TIRObject));
    //* Clear all
    ZeroMemory(@IRCreateObjectParameters, SizeOf(TIRCreateObjectParameters));
    //* No pre-processing of the image
    IRCreateObjectParameters.ProcessType := IR_PROCESS_TYPE_NONE;
    //* Create the object
    QueryPerformanceFrequency(PerfFreq);
    QueryPerformanceCounter(FreqCountStart);
    Error := ImageRecognition_CreateObject(PChar(Edit1.Text), IRObjectSearchIn, IRCreateObjectParameters);
    QueryPerformanceCounter(FreqCountEnd);
    ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Created object: $' + IntToHex(NativeInt(IRObjectSearchIn.ImageObject), 8) + ' ' + IntToStr(IRObjectSearchIn.OriginalDimensions.Width) + 'x' + IntToStr(IRObjectSearchIn.OriginalDimensions.Height) + ' in ' + mSec2Time(ProcessingTime));
        FreeImageLoadImage(Edit1.Text, Bitmap, Handle);
        Image1.Picture.Assign(Bitmap);
        FreeAndNil(Bitmap);
        //* Assign the search objects to OpenCL
        QueryPerformanceFrequency(PerfFreq);
        QueryPerformanceCounter(FreqCountStart);
        Error := ImageRecognition_OpenCLSetSearchIn(IROCLDevice, IRObjectSearchIn);
        QueryPerformanceCounter(FreqCountEnd);
        ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);
        if Error = IR_OK then begin
            Memo1.Lines.Add('Assigned the search in object to OpenCL in ' + mSec2Time(ProcessingTime));
        end else begin
            Memo1.Lines.Add('Error assigning the search in object to OpenCL.');
        end;
    end else begin
        Memo1.Lines.Add('Error creating object, error code: ' + IntToStr(Error));
    end;
    //* Clear all
    ZeroMemory(@IRObjectSearchFor, SizeOf(TIRObject));
    //* Clear all
    ZeroMemory(@IRCreateObjectParameters, SizeOf(TIRCreateObjectParameters));
    //* No pre-processing of the image
    IRCreateObjectParameters.ProcessType := IR_PROCESS_TYPE_NONE;
    //* Create the object
    QueryPerformanceFrequency(PerfFreq);
    QueryPerformanceCounter(FreqCountStart);
    Error := ImageRecognition_CreateObject(PChar(Edit2.Text), IRObjectSearchFor, IRCreateObjectParameters);
    QueryPerformanceCounter(FreqCountEnd);
    ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Created object: $' + IntToHex(NativeInt(IRObjectSearchFor.ImageObject), 8) + ' ' + IntToStr(IRObjectSearchFor.OriginalDimensions.Width) + 'x' + IntToStr(IRObjectSearchFor.OriginalDimensions.Height) + ' in ' + mSec2Time(ProcessingTime));
        FreeImageLoadImage(Edit2.Text, Bitmap, Handle);
        Image2.Picture.Assign(Bitmap);
        FreeAndNil(Bitmap);
        //* Assign the search objects to OpenCL
        QueryPerformanceFrequency(PerfFreq);
        QueryPerformanceCounter(FreqCountStart);
        Error := ImageRecognition_OpenCLSetSearchFor(IROCLDevice, IRObjectSearchFor);
        QueryPerformanceCounter(FreqCountEnd);
        ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);
        if Error = IR_OK then begin
            Memo1.Lines.Add('Assigned the search for object to OpenCL in ' + mSec2Time(ProcessingTime));
        end else begin
            Memo1.Lines.Add('Error assigning the search for object to OpenCL.');
        end;
    end else begin
        Memo1.Lines.Add('Error creating object, error code: ' + IntToStr(Error));
    end;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
    Error: Integer;
begin
    //* Search in object
    Error := ImageRecognition_FreeObject(IRObjectSearchIn);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Freed object: ' + IRObjectSearchIn.FileName);
        Image1.Picture.Assign(nil);
    end else begin
        Memo1.Lines.Add('Error freeing object, error code: ' + IntToStr(Error));
    end;
    //* Search for object
    Error := ImageRecognition_FreeObject(IRObjectSearchFor);
    if Error = IR_OK then begin
        Memo1.Lines.Add('Freed object: ' + IRObjectSearchFor.FileName);
        Image2.Picture.Assign(nil);
    end else begin
        Memo1.Lines.Add('Error freeing object, error code: ' + IntToStr(Error));
    end;
end;

function StatusCallback(Progress: Double; StatusType: Integer; User: Pointer): Integer; {$IFDEF MSWINDOWS}stdcall{$ELSE}cdecl{$ENDIF};
begin
    Form1.ProgressBar1.Position := Round(Progress);
    //* Return IR_ABORT_PROCESSING to abort the processing
    Result := IR_CONTINUE_PROCESSING;
end;

procedure TForm1.Button6Click(Sender: TObject);
var
    Parameters: TIRProcessParameters;
    Results: TIRResult;
    Error: Integer;
    PerfFreq, FreqCountStart, FreqCountEnd: Int64;
    ProcessingTime: Integer;
    PMatch: PIRResultMatches;
    i: Integer;
begin
    ProgressBar1.Position := 0;

    //* Clear all
    ZeroMemory(@Parameters, SizeOf(TIRProcessParameters));

    //* We do an "exact" search
    Parameters.CompareType := IR_COMPARE_TYPE_RELATIVE_ARGB_ALPHA; //* Not needed as the OpenCL device is already initialized for a compare type
    //* Allow multiple matches
    Parameters.MultipleMatchesMax := 255;
    //* Number of pixels that may differ completely
    Parameters.DifferenceTolerance := Trunc(IRObjectSearchFor.OriginalDimensions.Width * IRObjectSearchFor.OriginalDimensions.Height * (TrackBar9.Position / 100));
    //* A callback to get notified about the progress
    //* Note that this can significantly reduce performance
    Parameters.StatusCallback := StatusCallback;

    //* Filter out matches that are duplicates - their X-Y position difference is less than 10 pixels
    Parameters.MultipleMatchesFilterTresholdX := 10;
    Parameters.MultipleMatchesFilterTresholdY := 10;

    Parameters.ARGBATolerance := StrToInt(Edit3.Text);
    Parameters.ARGBRTolerance := StrToInt(Edit4.Text);
    Parameters.ARGBGTolerance := StrToInt(Edit5.Text);
    Parameters.ARGBBTolerance := StrToInt(Edit6.Text);

    QueryPerformanceFrequency(PerfFreq);
    QueryPerformanceCounter(FreqCountStart);

    //* Perform the search
    //for i := 0 to 999 do
    Error := ImageRecognition_OpenCLPerformSearch(IROCLDevice, Parameters, Results, nil);

    QueryPerformanceCounter(FreqCountEnd);
    ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);

    if Error = IR_OK then begin
        //* The processing completed normally
    end else begin
        Memo1.Lines.Add('Error while processing, error code: ' + IntToStr(Error));
    end;
    if Results.Success then begin
        Memo1.Lines.Add('Successful search, match count: ' + IntToStr(Results.MatchCount));
        //* Create a rectangle sorrounding the matching part of the picture
        Image1.Canvas.Pen.Color := clRed;
        Image1.Canvas.Pen.Width := 5;
        Image1.Canvas.Brush.Style := bsSolid;
        Image1.Canvas.Brush.Color := clRed;
        //* Point to the first match
        PMatch := Results.Matches;
        //* Iterate over the matches
        for i := 0 to Results.MatchCount - 1 do begin
            Memo1.Lines.Add('Found at position: '
                + IntToStr(PMatch.Position.X)
                + 'x'
                + IntToStr(PMatch.Position.Y)
                + ' difference: '
                + IntToStr(PMatch.Difference));
            Image1.Canvas.MoveTo(PMatch.Position.X, PMatch.Position.Y);
            Image1.Canvas.LineTo(PMatch.Position.X + PMatch.Dimensions.Width, PMatch.Position.Y);
            Image1.Canvas.LineTo(PMatch.Position.X + PMatch.Dimensions.Width, PMatch.Position.Y + PMatch.Dimensions.Height);
            Image1.Canvas.LineTo(PMatch.Position.X, PMatch.Position.Y + PMatch.Dimensions.Height);
            Image1.Canvas.LineTo(PMatch.Position.X, PMatch.Position.Y);
            //* Point to the next match
            Inc(PMatch);
        end;
        Memo1.Lines.Add('Processing time: ' + mSec2Time(ProcessingTime));
        Image1.Update;
        Image1.Invalidate;
        //* After finished with the result object free the allocated memory
        ImageRecognition_FreeResult(Results);
    end else begin
        Memo1.Lines.Add('No match found.');
    end;
end;

procedure TForm1.Button7Click(Sender: TObject);
var
    ComputeCoreCount: Integer;
    PerfFreq, FreqCountStart, FreqCountEnd: Int64;
    ProcessingTime: Integer;
begin
    QueryPerformanceFrequency(PerfFreq);
    QueryPerformanceCounter(FreqCountStart);
    //* An RTX 2060 has 1920 CUDA cores, less values will result a lower performance if there are more compute cores available than what is specified here. So if unsure use larger values.
    //* Ideal value is available cores * n, exact value gives the best performance.
    case ComboBoxCores.ItemIndex of
        0: ComputeCoreCount := 1920;  //* RTX 2060
        1: ComputeCoreCount := 2560;  //* RTX 2070 Super
        2: ComputeCoreCount := 4352;  //* RTX 2080
        3: ComputeCoreCount := 8704;  //* RTX 3080
        4: ComputeCoreCount := 10240; //* RTX 3080 Ti
        5: ComputeCoreCount := 10496; //* RTX 3090
        6: ComputeCoreCount := 3072;  //* RTX 4060
        7: ComputeCoreCount := 4352;  //* RTX 4060 Ti
        8: ComputeCoreCount := 5888;  //* RTX 4070
        9: ComputeCoreCount := 7680;  //* RTX 4070 Ti
        10: ComputeCoreCount := 7168;  //* RTX 4070 SUPER
        11: ComputeCoreCount := 8448;  //* RTX 4070 Ti SUPER
        12: ComputeCoreCount := 9728;  //* RTX 4080
        13: ComputeCoreCount := 10240; //* RTX 4080 SUPER
        14: ComputeCoreCount := 14592; //* RTX 4090 D
        15: ComputeCoreCount := 16384; //* RTX 4090
        16: ComputeCoreCount := 2560;  //* RTX 5050
        17: ComputeCoreCount := 3840;  //* RTX 5060
        18: ComputeCoreCount := 4608;  //* RTX 5060 Ti
        19: ComputeCoreCount := 6144;  //* RTX 5070
        20: ComputeCoreCount := 8960;  //* RTX 5070 Ti
        21: ComputeCoreCount := 10752; //* RTX 5080
        22: ComputeCoreCount := 21760; //* RTX 5090
    else
        ComputeCoreCount := 1920;
    end;
    IROCLDevice := ImageRecognition_OpenCLInit(ComboBoxOpenCLPlatform.ItemIndex, ComboBoxOpenCLDevice.ItemIndex, IR_COMPARE_TYPE_RELATIVE_ARGB_ALPHA, False, ComputeCoreCount);
    QueryPerformanceCounter(FreqCountEnd);
    ProcessingTime := Round((FreqCountEnd - FreqCountStart) / PerfFreq * 1000);
    if Assigned(IROCLDevice.Instance) then begin
        Memo1.Lines.Add('Successfully initialized OpenCL device ' + ComboBoxOpenCLDevice.Items[ComboBoxOpenCLDevice.ItemIndex] + ' in ' + mSec2Time(ProcessingTime));
        ComboBoxOpenCLPlatform.Enabled := False;
        ComboBoxOpenCLDevice.Enabled := False;
        ComboBoxCores.Enabled := False;
        Button7.Enabled := False;
    end else begin
        MessageDlg('Error while initializing OpenCL device.', mtError, [mbCancel], 0);
    end;
end;

procedure TForm1.ComboBoxOpenCLPlatformSelect(Sender: TObject);
begin
    ListOpenCLPlatformDevices(ComboBoxOpenCLPlatform.ItemIndex);
    if ComboBoxOpenCLDevice.Items.Count > 0 then begin
        ComboBoxOpenCLDevice.ItemIndex := 0;
    end;
end;

procedure TForm1.Edit3Exit(Sender: TObject);
begin
    TrackBar1.Position := StrToInt(Edit3.Text);
end;

procedure TForm1.Edit4Exit(Sender: TObject);
begin
    TrackBar2.Position := StrToInt(Edit4.Text);
end;

procedure TForm1.Edit5Exit(Sender: TObject);
begin
    TrackBar3.Position := StrToInt(Edit5.Text);
end;

procedure TForm1.Edit6Exit(Sender: TObject);
begin
    TrackBar4.Position := StrToInt(Edit6.Text);
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
    Edit3.Text := IntToStr(TrackBar1.Position);
end;

procedure TForm1.TrackBar2Change(Sender: TObject);
begin
    Edit4.Text := IntToStr(TrackBar2.Position);
end;

procedure TForm1.TrackBar3Change(Sender: TObject);
begin
    Edit5.Text := IntToStr(TrackBar3.Position);
end;

procedure TForm1.TrackBar4Change(Sender: TObject);
begin
    Edit6.Text := IntToStr(TrackBar4.Position);
end;

procedure TForm1.TrackBar9Change(Sender: TObject);
begin
    Label14.Caption := IntToStr(TrackBar9.Position) + ' %';
end;

end.
